package data.hullmods;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.combat.BaseHullMod;
import com.fs.starfarer.api.combat.CombatEngineAPI;
import com.fs.starfarer.api.combat.FluxTrackerAPI;
import com.fs.starfarer.api.combat.MutableShipStatsAPI;
import com.fs.starfarer.api.combat.ShipAPI;

public class FluxOverdrive extends BaseHullMod {

    // Change this to be whatever the prefix to your ship IDs will be
    private static final String FACTION_PREFIX = "Fox_";
    private CombatEngineAPI engine;

    @Override
    public boolean isApplicableToShip(ShipAPI ship) {
        return (ship.getHullSpec().getHullId().startsWith(FACTION_PREFIX));
    }

    @Override
    public void advanceInCombat(ShipAPI ship, float amount) {

        if (engine != Global.getCombatEngine()) {
            engine = Global.getCombatEngine();
        }

        if (!engine.isEntityInPlay(ship) || engine.isPaused() || !ship.isAlive()) {
            return;
        }

        MutableShipStatsAPI stats = ship.getMutableStats();
        String id = "FluxOverdrive";

        FluxTrackerAPI flux = ship.getFluxTracker();
        final float fluxLevel = flux.getCurrFlux() / flux.getMaxFlux();
        float speedBonus = 1f, accelBonus = 1f;

        // Bonus is higher if ship is currently venting/overloaded
        if (flux.isOverloadedOrVenting()) {
            speedBonus = 2.5f;
            accelBonus = 0.5f;
        }

        stats.getMaxSpeed().modifyFlat(id, 25f * speedBonus * fluxLevel);
        stats.getMaxTurnRate().modifyFlat(id, 5f * (accelBonus/2f) * fluxLevel);
        stats.getMaxTurnRate().modifyPercent(id, 100f * (accelBonus/2f) * fluxLevel);
        stats.getAcceleration().modifyPercent(id, 150f * accelBonus * fluxLevel);
        stats.getDeceleration().modifyPercent(id, 150f * accelBonus * fluxLevel);
        stats.getTurnAcceleration().modifyFlat(id, 10f * (speedBonus/1.5f) * fluxLevel);
        stats.getTurnAcceleration().modifyPercent(id, 150f * (speedBonus/1.5f) * fluxLevel);
    }
}
