package data.shipsystems.scripts;

import com.fs.starfarer.api.combat.MutableShipStatsAPI;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.plugins.ShipSystemStatsScript;

public class PowerCruiseStats implements ShipSystemStatsScript {

    @Override
    public void apply(MutableShipStatsAPI stats, String id, State state, float effectLevel) {
        if (state == ShipSystemStatsScript.State.OUT) {
            stats.getMaxSpeed().unmodify(id); // to slow down ship to its regular top speed while powering drive down
            stats.getMaxTurnRate().unmodify(id);
        } else {

            ShipAPI ship = (ShipAPI) stats.getEntity();


            if (null != ship) {
                if (ship.getFluxTracker().isEngineBoostActive()) {
                    stats.getMaxSpeed().modifyFlat(id, -100f + 220f * effectLevel);
                } else {
                    stats.getMaxSpeed().modifyFlat(id, -100f + 270 * effectLevel);
                }
            }
            stats.getAcceleration().modifyPercent(id, 0.1f);
            stats.getDeceleration().modifyPercent(id, 25f);
            stats.getTurnAcceleration().modifyPercent(id, 30f);
            stats.getMaxTurnRate().modifyPercent(id, 10f);

        }
    }

    @Override
    public void unapply(MutableShipStatsAPI stats, String id) {
        stats.getMaxSpeed().unmodify(id);
        stats.getMaxTurnRate().unmodify(id);
        stats.getTurnAcceleration().unmodify(id);
        stats.getAcceleration().unmodify(id);
        stats.getDeceleration().unmodify(id);
    }

    @Override
    public StatusData getStatusData(int index, State state, float effectLevel) {
        if (index == 0) {
            return new StatusData("improved speed, thruster power redirected", false);
        } else if (index == 1) {
            return new StatusData("200% top speed", false);
        }
        return null;
    }
}
