package com.fs.starfarer.api.combat;

import com.fs.starfarer.api.combat.ShipAPI.HullSize;
import com.fs.starfarer.api.fleet.FleetMemberAPI;

/**
 * Note: the effect class is instantiated once per application session.
 * Storing campaign data in members of an implementing class is a bad idea,
 * use SectorAPI.getPersistentData() instead.
 * @author Alex Mosolov
 *
 * Copyright 2015 Fractal Softworks, LLC
 */
public interface HullModEffect {
	void applyEffectsBeforeShipCreation(HullSize hullSize, MutableShipStatsAPI stats, String id);
	
	/**
	 * Effects applied here should NOT affect ship stats as this does not get called from the campaign.
	 * Apply stat changes in applyEffectsBeforeShipCreation() instead, as that does affect the campaign.
	 * @param ship
	 * @param id
	 */
	void applyEffectsAfterShipCreation(ShipAPI ship, String id);
	String getDescriptionParam(int index, HullSize hullSize);
	
	boolean isApplicableToShip(ShipAPI ship);
	
	
	/**
	 * Not called while paused.
	 * But, called when the fleet data needs to be re-synced,
	 * with amount=0 (such as if, say, a fleet member is moved around.
	 * in the fleet screen.)
	 * @param member
	 * @param amount
	 */
	void advanceInCampaign(FleetMemberAPI member, float amount);
	
	/**
	 * Not called while paused.
	 * @param ship
	 * @param amount
	 */
	void advanceInCombat(ShipAPI ship, float amount);
}
