package com.fs.starfarer.api.impl.campaign;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.log4j.Logger;

import com.fs.starfarer.api.EveryFrameScript;
import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.BaseCampaignEventListener;
import com.fs.starfarer.api.campaign.BattleAPI;
import com.fs.starfarer.api.campaign.CampaignFleetAPI;
import com.fs.starfarer.api.campaign.PlayerMarketTransaction;
import com.fs.starfarer.api.campaign.RepLevel;
import com.fs.starfarer.api.campaign.SectorAPI;
import com.fs.starfarer.api.campaign.SectorEntityToken;
import com.fs.starfarer.api.campaign.econ.MarketAPI;
import com.fs.starfarer.api.campaign.econ.SubmarketAPI;
import com.fs.starfarer.api.campaign.rules.MemoryAPI;
import com.fs.starfarer.api.characters.PersonAPI;
import com.fs.starfarer.api.impl.campaign.events.BaseEventPlugin.MarketFilter;
import com.fs.starfarer.api.impl.campaign.fleets.LionsGuardFleetManager;
import com.fs.starfarer.api.impl.campaign.fleets.PatrolFleetManager;
import com.fs.starfarer.api.impl.campaign.ids.Factions;
import com.fs.starfarer.api.impl.campaign.ids.MemFlags;
import com.fs.starfarer.api.impl.campaign.shared.SharedData;
import com.fs.starfarer.api.util.IntervalUtil;
import com.fs.starfarer.api.util.Misc;

public class CoreScript extends BaseCampaignEventListener implements EveryFrameScript {

	public static Logger log = Global.getLogger(CoreScript.class);
	
	public static final String SHARED_DATA_KEY = "core_CEFSSharedDataKey";
	
	private SharedData shared;
	
	private IntervalUtil timer = new IntervalUtil(0.5f, 1.5f);
	private Set<String> marketsWithAssignedPatrolScripts = new HashSet<String>();
	
	public CoreScript() {
		super(true);
//		shared = new SharedData();
//		Global.getSector().getPersistentData().put(SHARED_DATA_KEY, shared);
		shared = SharedData.getData();
	}

	private boolean firstFrame = true;
	public void advance(float amount) {
		SectorAPI sector = Global.getSector();
		playRepChangeSoundsIfNeeded();
		
		if (sector.isPaused()) {
			return;
		}
		
		if (firstFrame) {
			assignPatrolSpawningScripts();
			firstFrame = false;
		}
		
		float days = sector.getClock().convertToDays(amount);
		shared.advance(amount);
		
		timer.advance(days);
		if (timer.intervalElapsed()) {
			assignPatrolSpawningScripts();
		}
	}
	
	
	private void playRepChangeSoundsIfNeeded() {
		if (deltaFaction != null) {
			if (highestDelta > 0) {
				Global.getSoundPlayer().playUISound("ui_rep_raise", 1, 1);
			} else if (highestDelta < 0) {
				Global.getSoundPlayer().playUISound("ui_rep_drop", 1, 1);
			}
		}
		
		highestDelta = 0f;
		deltaFaction = null;
	}

	
	private float highestDelta = 0f;
	private String deltaFaction = null;
	@Override
	public void reportPlayerReputationChange(String faction, float delta) {
		super.reportPlayerReputationChange(faction, delta);
		if (Math.abs(delta) > Math.abs(highestDelta)) {
			highestDelta = delta;
			deltaFaction = faction;
		}
	}
	

	@Override
	public void reportPlayerReputationChange(PersonAPI person, float delta) {
		super.reportPlayerReputationChange(person, delta);
		if (Math.abs(delta) > Math.abs(highestDelta)) {
			highestDelta = delta;
			deltaFaction = person.getFaction().getId();
		}
	}


	public void assignPatrolSpawningScripts() {
		for (MarketAPI market : Global.getSector().getEconomy().getMarketsCopy()) {
			if (SharedData.getData().getMarketsWithoutPatrolSpawn().contains(market.getId())) continue;
			
			//if (market.getFactionId().equals(Factions.PIRATES)) continue;
			if (market.getFaction().getCustom().optBoolean(Factions.CUSTOM_NO_PATROLS)) continue;
			
			String id = market.getId();
			
			
			if (marketsWithAssignedPatrolScripts.contains(id)) continue;
			marketsWithAssignedPatrolScripts.add(id);
			
			if (id.equals("sindria")) {
				LionsGuardFleetManager script = new LionsGuardFleetManager(market);
				SectorEntityToken entity = market.getPrimaryEntity();
				entity.addScript(script);
			}
			
			
			PatrolFleetManager script = new PatrolFleetManager(market);
			SectorEntityToken entity = market.getPrimaryEntity();
			entity.addScript(script);
			log.info("Added patrol fleet spawning script to market [" + market.getName() + "]");
			
//			MercAndPirateFleetManager pirateScript = new MercAndPirateFleetManager(market);
//			entity.addScript(pirateScript);
//			log.info("Added pirate fleet spawning script to market [" + market.getName() + "]");
			
		}
	}
	
	public boolean isDone() {
		return false;
	}

	public boolean runWhilePaused() {
		return true;
	}


	@Override
	public void reportPlayerMarketTransaction(PlayerMarketTransaction transaction) {
		super.reportPlayerMarketTransaction(transaction);
		
		//SharedData.getData().getPlayerActivityTracker().updateLastVisit(transaction.getMarket());
		
		SubmarketAPI sub = transaction.getSubmarket();
		if (sub.getPlugin().isParticipatesInEconomy()) {
			SharedData.getData().getPlayerActivityTracker().getPlayerTradeData(sub).addTransaction(transaction);
		}
	}

	@Override
	public void reportPlayerOpenedMarket(MarketAPI market) {
		super.reportPlayerOpenedMarket(market);
		SharedData.getData().getPlayerActivityTracker().updateLastVisit(market);
	}


	@Override
	public void reportBattleOccurred(CampaignFleetAPI primaryWinner, BattleAPI battle) {
		if (!battle.isPlayerInvolved()) return;

		for (final CampaignFleetAPI otherFleet : battle.getNonPlayerSideSnapshot()) {
			MemoryAPI memory = otherFleet.getMemoryWithoutUpdate();
			//if (!playerFleet.isTransponderOn()) {
				Misc.setFlagWithReason(memory, MemFlags.MEMORY_KEY_MAKE_HOSTILE_WHILE_TOFF, "battle", true, 7f + (float) Math.random() * 7f);
			//}
			otherFleet.addScript(new TOffAlarm(otherFleet));
			
			
			float fpLost = Misc.getSnapshotFPLost(otherFleet);
	
			List<MarketAPI> markets = Misc.findNearbyLocalMarkets(otherFleet,
					Global.getSettings().getFloat("sensorRangeMax") + 500f,
				new MarketFilter() {
					public boolean acceptMarket(MarketAPI market) {
						return market.getFaction().isAtWorst(otherFleet.getFaction(), RepLevel.COOPERATIVE);
					}
				});
			
			for (MarketAPI market : markets) {
				MemoryAPI mem = market.getMemoryWithoutUpdate();
				float expire = fpLost;
				if (mem.contains(MemFlags.MEMORY_KEY_PLAYER_HOSTILE_ACTIVITY_NEAR_MARKET)) {
					expire += mem.getExpire(MemFlags.MEMORY_KEY_PLAYER_HOSTILE_ACTIVITY_NEAR_MARKET); 
				}
				if (expire > 180) expire = 180;
				if (expire > 0) {
					mem.set(MemFlags.MEMORY_KEY_PLAYER_HOSTILE_ACTIVITY_NEAR_MARKET, true, expire);
				}
			}
		}
	}

	
}





