package com.fs.starfarer.api.impl.campaign;

import java.util.ArrayList;
import java.util.EnumSet;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.Set;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.BattleAPI;
import com.fs.starfarer.api.campaign.CampaignFleetAPI;
import com.fs.starfarer.api.campaign.CargoAPI;
import com.fs.starfarer.api.campaign.CargoStackAPI;
import com.fs.starfarer.api.campaign.CombatDamageData;
import com.fs.starfarer.api.campaign.EngagementResultForFleetAPI;
import com.fs.starfarer.api.campaign.FactionAPI;
import com.fs.starfarer.api.campaign.FleetDataAPI;
import com.fs.starfarer.api.campaign.FleetEncounterContextPlugin;
import com.fs.starfarer.api.campaign.InteractionDialogAPI;
import com.fs.starfarer.api.campaign.CampaignEventListener.FleetDespawnReason;
import com.fs.starfarer.api.campaign.CargoAPI.CrewXPLevel;
import com.fs.starfarer.api.campaign.CombatDamageData.DamageToFleetMember;
import com.fs.starfarer.api.campaign.CombatDamageData.DealtByFleetMember;
import com.fs.starfarer.api.campaign.FleetEncounterContextPlugin.DataForEncounterSide.OfficerEngagementData;
import com.fs.starfarer.api.campaign.ai.ModularFleetAIAPI;
import com.fs.starfarer.api.campaign.ai.CampaignFleetAIAPI.PursuitOption;
import com.fs.starfarer.api.campaign.econ.CommoditySpecAPI;
import com.fs.starfarer.api.characters.MutableCharacterStatsAPI;
import com.fs.starfarer.api.characters.OfficerDataAPI;
import com.fs.starfarer.api.characters.PersonAPI;
import com.fs.starfarer.api.combat.DeployedFleetMemberAPI;
import com.fs.starfarer.api.combat.EngagementResultAPI;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.ShipVariantAPI;
import com.fs.starfarer.api.combat.WeaponAPI;
import com.fs.starfarer.api.combat.WeaponAPI.WeaponType;
import com.fs.starfarer.api.fleet.CrewCompositionAPI;
import com.fs.starfarer.api.fleet.FleetGoal;
import com.fs.starfarer.api.fleet.FleetMemberAPI;
import com.fs.starfarer.api.impl.campaign.CoreReputationPlugin.RepActionEnvelope;
import com.fs.starfarer.api.impl.campaign.CoreReputationPlugin.RepActions;
import com.fs.starfarer.api.impl.campaign.ids.Commodities;
import com.fs.starfarer.api.impl.campaign.ids.Stats;
import com.fs.starfarer.api.loading.VariantSource;
import com.fs.starfarer.api.util.Misc;
import com.fs.starfarer.api.util.WeightedRandomPicker;

public class FleetEncounterContext implements FleetEncounterContextPlugin {
	
	protected List<DataForEncounterSide> sideData = new ArrayList<DataForEncounterSide>();
	protected boolean engagedInHostilities = false;
	protected boolean playerOnlyRetreated = true;
	protected boolean playerPursued = false;
	protected boolean playerDidSeriousDamage = false;
	protected BattleAPI battle;
	protected boolean otherFleetHarriedPlayer = false;
	protected boolean ongoingBattle = false;
	
	protected CombatDamageData runningDamageTotal = null;
	
	
	public BattleAPI getBattle() {
		return battle;
	}

	public void setBattle(BattleAPI battle) {
		this.battle = battle;
	}

	public DataForEncounterSide getDataFor(CampaignFleetAPI participantOrCombined) {
//		if (participant == null) {
//			System.out.println("wtfffff");
//		}
		CampaignFleetAPI combined = battle.getCombinedFor(participantOrCombined);
		if (combined == null) {
			return new DataForEncounterSide(participantOrCombined);
		}
		
		for (DataForEncounterSide curr : sideData) {
			if (curr.getFleet() == combined) return curr;
		}
		DataForEncounterSide dfes = new DataForEncounterSide(combined);
		sideData.add(dfes);
		
		return dfes;
	}
	
	public DataForEncounterSide getWinnerData() {
		for (DataForEncounterSide curr : sideData) {
			if (!curr.disengaged()) {
				return curr;
			}
		}
		return null;
	}
	
	public DataForEncounterSide getLoserData() {
		for (DataForEncounterSide curr : sideData) {
			if (curr.disengaged()) {
				return curr;
			}
		}
		return null;
	}
	
	public boolean isEngagedInHostilities() {
		return engagedInHostilities;
	}
	
	public void setEngagedInHostilities(boolean engagedInHostilities) {
		this.engagedInHostilities = engagedInHostilities;
	}
	
	public void setOtherFleetHarriedPlayer(boolean otherFleetHarriedPlayer) {
		this.otherFleetHarriedPlayer = otherFleetHarriedPlayer;
	}
	
	public boolean isOtherFleetHarriedPlayer() {
		return otherFleetHarriedPlayer;
	}

	protected void updateDeployedMap(EngagementResultForFleetAPI result) {
		DataForEncounterSide data = getDataFor(result.getFleet());
		data.getMemberToDeployedMap().clear();
		
		List<DeployedFleetMemberAPI> deployed = result.getAllEverDeployedCopy();
		if (deployed != null && !deployed.isEmpty()) {
			for (DeployedFleetMemberAPI dfm : deployed) {
				if (dfm.getMember() != null) {
					data.getMemberToDeployedMap().put(dfm.getMember(), dfm);
				}
			}
		}
	}
	
	public void processEngagementResults(EngagementResultAPI result) {
		engagedInHostilities = true;
		
		// the fleets we get back here are combined fleets from the BattleAPI,
		// NOT the actual fleets involved, even if it's a 1 vs 1 battle.
		EngagementResultForFleetAPI winnerResult = result.getWinnerResult();
		EngagementResultForFleetAPI loserResult = result.getLoserResult();

		// only happens for combat where player is involved
		CombatDamageData currDamageData = result.getLastCombatDamageData();
		if (currDamageData != null) {
			if (runningDamageTotal == null) {
				runningDamageTotal = currDamageData;
			} else {
				runningDamageTotal.add(currDamageData);
			}
			computeFPHullDamage();
		}
		
		//if (winnerResult.getFleet().isPlayerFleet() || loserResult.getFleet().isPlayerFleet()) {
		if (battle.isPlayerInvolved()) {
			Global.getSector().reportPlayerEngagement(result);
		}
		
		updateDeployedMap(winnerResult);
		updateDeployedMap(loserResult);
		
		//result.applyToFleets();
		applyResultToFleets(result);

		//if (winnerResult.getFleet().isPlayerFleet() && winnerResult.getGoal() != FleetGoal.ESCAPE) {
		if (battle.isPlayerSide(winnerResult) && winnerResult.getGoal() != FleetGoal.ESCAPE) {
			playerOnlyRetreated = false;
			if (loserResult.getGoal() == FleetGoal.ESCAPE) {
				playerPursued = true;
			}
		//} else if (loserResult.getFleet().isPlayerFleet() && loserResult.getGoal() != FleetGoal.ESCAPE) {
		} else if (battle.isPlayerSide(loserResult) && loserResult.getGoal() != FleetGoal.ESCAPE) {
			playerOnlyRetreated = false;
			if (winnerResult.getGoal() == FleetGoal.ESCAPE) {
				playerPursued = true;
			}
		}
		
		DataForEncounterSide winnerData = getDataFor(winnerResult.getFleet());
		DataForEncounterSide loserData = getDataFor(loserResult.getFleet());
		
		winnerData.setWonLastEngagement(true);
		loserData.setWonLastEngagement(false);
		
		winnerData.setDidEnoughToDisengage(true);
		float damageInFP = 0f;
		for (FleetMemberAPI member : winnerResult.getDisabled()) {
			damageInFP += member.getFleetPointCost();
		}
		for (FleetMemberAPI member : winnerResult.getDestroyed()) {
			damageInFP += member.getFleetPointCost();
		}
		for (FleetMemberAPI member : winnerResult.getRetreated()) {
			damageInFP += member.getFleetPointCost();
		}
		
		float remaining = 0f;
		for (FleetMemberAPI member : winnerResult.getFleet().getFleetData().getCombatReadyMembersListCopy()) {
			remaining += member.getFleetPointCost();
		}
		loserData.setDidEnoughToDisengage(damageInFP >= remaining);
		
		
		winnerData.setLastGoal(winnerResult.getGoal());
		loserData.setLastGoal(loserResult.getGoal());
		
		winnerData.getDeployedInLastEngagement().clear();
		winnerData.getRetreatedFromLastEngagement().clear();
		winnerData.getInReserveDuringLastEngagement().clear();
		winnerData.getDisabledInLastEngagement().clear();
		winnerData.getDestroyedInLastEngagement().clear();
		winnerData.getDeployedInLastEngagement().addAll(winnerResult.getDeployed());
		winnerData.getRetreatedFromLastEngagement().addAll(winnerResult.getRetreated());
		winnerData.getInReserveDuringLastEngagement().addAll(winnerResult.getReserves());
		winnerData.getDisabledInLastEngagement().addAll(winnerResult.getDisabled());
		winnerData.getDestroyedInLastEngagement().addAll(winnerResult.getDestroyed());
		
		loserData.getDeployedInLastEngagement().clear();
		loserData.getRetreatedFromLastEngagement().clear();
		loserData.getInReserveDuringLastEngagement().clear();
		loserData.getDisabledInLastEngagement().clear();
		loserData.getDestroyedInLastEngagement().clear();
		loserData.getDeployedInLastEngagement().addAll(loserResult.getDeployed());
		loserData.getRetreatedFromLastEngagement().addAll(loserResult.getRetreated());
		loserData.getInReserveDuringLastEngagement().addAll(loserResult.getReserves());
		loserData.getDisabledInLastEngagement().addAll(loserResult.getDisabled());
		loserData.getDestroyedInLastEngagement().addAll(loserResult.getDestroyed());
		
		for (FleetMemberAPI member : loserResult.getDestroyed()) {
			loserData.addOwn(member, Status.DESTROYED);
		}
		
		for (FleetMemberAPI member : loserResult.getDisabled()) {
			loserData.addOwn(member, Status.DISABLED);
		}
		
		for (FleetMemberAPI member : winnerResult.getDestroyed()) {
			winnerData.addOwn(member, Status.DESTROYED);
		}
		
		for (FleetMemberAPI member : winnerResult.getDisabled()) {
			winnerData.addOwn(member, Status.DISABLED);
		}
		
		//if (winnerResult.getFleet().isPlayerFleet()) {
		if (result.getWinnerResult().getAllEverDeployedCopy() != null) {
			tallyOfficerTime(winnerData, winnerResult);
		}
		//} else if (loserResult.getFleet().isPlayerFleet()) {
		if (result.getLoserResult().getAllEverDeployedCopy() != null) {
			tallyOfficerTime(loserData, loserResult);
		}
		
		// important, so that in-combat Ship objects can be garbage collected.
		// Probably some combat engine references in there, too. 
		winnerResult.resetAllEverDeployed();
		getDataFor(winnerResult.getFleet()).getMemberToDeployedMap().clear();
		loserResult.resetAllEverDeployed();
		getDataFor(loserResult.getFleet()).getMemberToDeployedMap().clear();
		
		
		// moved from applyPostEngagementResult
		for (FleetMemberAPI member : loserResult.getDestroyed()) {
			winnerData.addEnemy(member, Status.DESTROYED);
		}
		for (FleetMemberAPI member : loserResult.getDisabled()) {
			winnerData.addEnemy(member, Status.DISABLED);
		}
		
		
		FleetGoal winnerGoal = winnerResult.getGoal();
		FleetGoal loserGoal = loserResult.getGoal();
		boolean totalWin = loserData.getFleet().getFleetData().getMembersListCopy().isEmpty();
		boolean playerOut = result.isPlayerOutBeforeEnd();
		
		if (playerOut) {
			FleetGoal playerGoal = null;
			FleetGoal otherGoal = null;
			if (battle.isPlayerSide(battle.getSideFor(winnerResult.getFleet()))) {
				playerGoal = winnerGoal;
				otherGoal = loserGoal;
			} else {
				playerGoal = loserGoal;
				otherGoal = winnerGoal;
			}
			if (playerGoal == FleetGoal.ATTACK) {
				if (otherGoal == FleetGoal.ATTACK) {
					if (winnerResult.isPlayer()) {
						lastOutcome = EngagementOutcome.BATTLE_PLAYER_OUT_FIRST_WIN;
					} else {
						lastOutcome = EngagementOutcome.BATTLE_PLAYER_OUT_FIRST_LOSS;
					}
				} else {
					if (winnerResult.isPlayer()) {
						lastOutcome = EngagementOutcome.PURSUIT_PLAYER_OUT_FIRST_WIN;
					} else {
						lastOutcome = EngagementOutcome.PURSUIT_PLAYER_OUT_FIRST_LOSS;
					}
				}
			} else {
				if (winnerResult.isPlayer()) {
					lastOutcome = EngagementOutcome.ESCAPE_PLAYER_OUT_FIRST_WIN;
				} else {
					lastOutcome = EngagementOutcome.ESCAPE_PLAYER_OUT_FIRST_LOSS;
				}
			}
		} else {
			if (totalWin && winnerData.getFleet().getFleetData().getMembersListCopy().isEmpty()) {
				lastOutcome = EngagementOutcome.MUTUAL_DESTRUCTION;
			} else {
				if (battle.isPlayerSide(battle.getSideFor(winnerResult.getFleet()))) {
					if (winnerGoal == FleetGoal.ATTACK && loserGoal == FleetGoal.ATTACK) {
						if (totalWin) {
							lastOutcome = EngagementOutcome.BATTLE_PLAYER_WIN_TOTAL;
						} else {
							lastOutcome = EngagementOutcome.BATTLE_PLAYER_WIN;
						}
					} else if (winnerGoal == FleetGoal.ESCAPE) {
						if (totalWin) {
							lastOutcome = EngagementOutcome.ESCAPE_PLAYER_WIN_TOTAL;
						} else {
							lastOutcome = EngagementOutcome.ESCAPE_PLAYER_WIN;
						}
					} else if (loserGoal == FleetGoal.ESCAPE) {
						if (totalWin) {
							lastOutcome = EngagementOutcome.ESCAPE_ENEMY_LOSS_TOTAL;
						} else {
							lastOutcome = EngagementOutcome.ESCAPE_ENEMY_SUCCESS;
						}
					}
				} else {
					if (winnerGoal == FleetGoal.ATTACK && loserGoal == FleetGoal.ATTACK) {
						if (totalWin) {
							lastOutcome = EngagementOutcome.BATTLE_ENEMY_WIN_TOTAL;
						} else {
							lastOutcome = EngagementOutcome.BATTLE_ENEMY_WIN;
						}
					} else if (winnerGoal == FleetGoal.ESCAPE) {
						if (totalWin) {
							lastOutcome = EngagementOutcome.ESCAPE_ENEMY_WIN_TOTAL;
						} else {
							lastOutcome = EngagementOutcome.ESCAPE_ENEMY_WIN;
						}
					} else if (loserGoal == FleetGoal.ESCAPE) {
						if (totalWin) {
							lastOutcome = EngagementOutcome.ESCAPE_PLAYER_LOSS_TOTAL;
						} else {
							lastOutcome = EngagementOutcome.ESCAPE_PLAYER_SUCCESS;
						}
					}
				}
			}
		}
		
		battle.uncombine();
		battle.genCombined();
	}
	
	protected void tallyOfficerTime(DataForEncounterSide data, EngagementResultForFleetAPI result) {
		float maxTime = 0f;
		for (DeployedFleetMemberAPI dfm : result.getAllEverDeployedCopy()) {
			float time = dfm.getShip().getFullTimeDeployed();
			if (time > maxTime) {
				maxTime = time;
			}
			PersonAPI person = dfm.getMember().getCaptain();
			CampaignFleetAPI source = battle.getSourceFleet(dfm.getMember());
			//if (data.getFleet().getFleetData().getOfficerData(person) == null) {
			if (source.getFleetData().getOfficerData(person) == null) {
				continue; // not an officer
			}
			OfficerEngagementData oed = data.getOfficerData().get(person);
			if (oed == null) {
				oed = new OfficerEngagementData(source);
				oed.person = person;
				data.getOfficerData().put(person, oed);
			}
			oed.timeDeployed += time;
		}
		data.setMaxTimeDeployed(data.getMaxTimeDeployed() + maxTime);
	}
	
	public PursueAvailability getPursuitAvailability(CampaignFleetAPI fleet, CampaignFleetAPI otherFleet) {
		DataForEncounterSide otherData = getDataFor(otherFleet);
		
		if (otherData.isWonLastEngagement()) return PursueAvailability.LOST_LAST_ENGAGEMENT;
		if (canOutrunOtherFleet(otherFleet, fleet)) return PursueAvailability.TOO_SLOW;
		if (fleet.getFleetData().getCombatReadyMembersListCopy().isEmpty()) return PursueAvailability.NO_READY_SHIPS;
		if (otherData.isDidEnoughToDisengage()) return PursueAvailability.TOOK_SERIOUS_LOSSES;
		return PursueAvailability.AVAILABLE;
	}
	
	public DisengageHarryAvailability getDisengageHarryAvailability(CampaignFleetAPI fleet, CampaignFleetAPI otherFleet) {
		DataForEncounterSide otherData = getDataFor(otherFleet);
		if (otherData.isWonLastEngagement()) return DisengageHarryAvailability.LOST_LAST_ENGAGEMENT;
		if (fleet.getFleetData().getCombatReadyMembersListCopy().isEmpty()) return DisengageHarryAvailability.NO_READY_SHIPS;
		return DisengageHarryAvailability.AVAILABLE;
	}
	
	public float getDeployCost(FleetMemberAPI member) {
		return member.getDeployCost();
	}
	
	protected boolean alreadyAdjustedRep = false;
	public boolean adjustPlayerReputation(InteractionDialogAPI dialog, String ffText) {
		if (alreadyAdjustedRep) return false;
		
		if (battle != null && battle.isPlayerInvolved() && engagedInHostilities) {
			alreadyAdjustedRep = true;
			
			boolean playerWon = didPlayerWinEncounter();
			List<CampaignFleetAPI> playerSide = battle.getPlayerSide();
			List<CampaignFleetAPI> enemySide = battle.getNonPlayerSide();
			
			CampaignFleetAPI pf = Global.getSector().getPlayerFleet();
			pf.setMoveDestination(pf.getLocation().x, pf.getLocation().y);
			
			// cases to cover: 1) player destroyed ships 2) player harried/harassed 3) player pursued
			// i.e. anything other than a non-destructive retreat, w/o an engagement
			boolean playerWasAggressive = playerDidSeriousDamage || !playerOnlyRetreated;
			RepActions action = null;
			if (engagedInHostilities) {
				action = RepActions.COMBAT_NO_DAMAGE_ESCAPE;
			}
			boolean knowsWhoPlayerIs = battle.knowsWhoPlayerIs(enemySide);
			if (playerPursued && playerWon) {
				if (knowsWhoPlayerIs) {
					action = RepActions.COMBAT_AGGRESSIVE;
				} else {
					action = RepActions.COMBAT_AGGRESSIVE_TOFF;
				}
			} else if (playerWasAggressive) {
				if (knowsWhoPlayerIs) {
					action = RepActions.COMBAT_NORMAL;
				} else {
					action = RepActions.COMBAT_NORMAL_TOFF;
				}
			}
			Set<String> seen = new HashSet<String>();
			if (action != null) {
				// use snapshot: ensure loss of reputation with factions if their fleets were destroyed
				for (CampaignFleetAPI enemy : battle.getSnapshotFor(enemySide)) {
					String factionId = enemy.getFaction().getId();
					if (seen.contains(factionId)) continue;
					seen.add(factionId);
					Global.getSector().adjustPlayerReputation(new RepActionEnvelope(action, null, dialog.getTextPanel()), factionId);
				}
			}
			
			//if (playerWon) {
				action = RepActions.COMBAT_HELP_MINOR;
				float playerFP = 0;
				float allyFP = 0;
				float enemyFP = 0;
				for (CampaignFleetAPI fleet : battle.getSnapshotFor(playerSide)) {
					for (FleetMemberAPI member : fleet.getFleetData().getSnapshot()) {
						if (fleet.isPlayerFleet()) {
							playerFP += member.getFleetPointCost();
						} else {
							allyFP += member.getFleetPointCost();
						}
					}
				}
				for (CampaignFleetAPI fleet : battle.getSnapshotFor(enemySide)) {
					for (FleetMemberAPI member : fleet.getFleetData().getSnapshot()) {
						enemyFP += member.getFleetPointCost();
					}
				}
				if (allyFP > enemyFP || !playerWon) {
					action = RepActions.COMBAT_HELP_MINOR;
				} else if (allyFP < enemyFP * 0.5f) {
					action = RepActions.COMBAT_HELP_CRITICAL;
				} else {
					action = RepActions.COMBAT_HELP_MAJOR;
				}
				
//				if (playerFPHullDamageToEnemies <= 0) {
//					action = null;
//				} else if (playerFPHullDamageToEnemies < allyFPHullDamageToEnemies * 0.1f) {
//					action = RepActions.COMBAT_HELP_MINOR;
//				}
				float f = computePlayerContribFraction();
				if (f <= 0) {
					action = null;
				} else if (f < 0.1f) {
					action = RepActions.COMBAT_HELP_MINOR;
				}
				
				if (action != null) {
					float totalDam = allyFPHullDamageToEnemies + playerFPHullDamageToEnemies;
					if (totalDam < 10) {
						action = RepActions.COMBAT_HELP_MINOR;
					} else if (totalDam < 20 && action == RepActions.COMBAT_HELP_CRITICAL) {
						action = RepActions.COMBAT_HELP_MAJOR;
					}
				}
				
				if (battle.isPlayerInvolvedAtStart() && action != null) {
					//action = RepActions.COMBAT_HELP_MINOR;
					action = null;
				}
//				if (leavingEarly) {
//					action = null;
//				}
				
				// rep increases
				seen.clear();
				for (CampaignFleetAPI ally : battle.getSnapshotFor(playerSide)) {
					if (ally.isPlayerFleet()) continue;
					
					String factionId = ally.getFaction().getId();
					if (seen.contains(factionId)) continue;
					seen.add(factionId);
					
					Float friendlyFPHull = playerFPHullDamageToAlliesByFaction.get(ally.getFaction());
					float threshold = 1f;
					if (action == RepActions.COMBAT_HELP_MAJOR) {
						threshold = 3f;
					} else if (action == RepActions.COMBAT_HELP_CRITICAL) {
						threshold = 7f;
					}
					if (friendlyFPHull != null && friendlyFPHull > threshold) {
						// can lose reputation with sides that didn't survive
						//Global.getSector().adjustPlayerReputation(new RepActionEnvelope(RepActions.COMBAT_FRIENDLY_FIRE, (friendlyFPHull - threshold), dialog.getTextPanel()), factionId);
					} else if (action != null && playerSide.contains(ally)) {
						// only gain reputation with factions whose fleets actually survived
						Global.getSector().adjustPlayerReputation(new RepActionEnvelope(action, null, dialog.getTextPanel()), factionId);
					}
				}
				
				
				// friendly fire rep decreases
				boolean first = true;
				seen.clear();
				for (CampaignFleetAPI ally : battle.getSnapshotFor(playerSide)) {
					if (ally.isPlayerFleet()) continue;
					
					String factionId = ally.getFaction().getId();
					if (seen.contains(factionId)) continue;
					seen.add(factionId);
					
					Float friendlyFPHull = playerFPHullDamageToAlliesByFaction.get(ally.getFaction());
					float threshold = 1f;
					if (action == RepActions.COMBAT_HELP_MAJOR) {
						threshold = 3f;
					} else if (action == RepActions.COMBAT_HELP_CRITICAL) {
						threshold = 7f;
					}
					if (friendlyFPHull != null && friendlyFPHull > threshold) {
						if (first && ffText != null) {
							first = false;
							dialog.getTextPanel().addParagraph(ffText);
						}
						// can lose reputation with sides that didn't survive
						Global.getSector().adjustPlayerReputation(new RepActionEnvelope(RepActions.COMBAT_FRIENDLY_FIRE, (friendlyFPHull - threshold), dialog.getTextPanel()), factionId);
					} else if (action != null && playerSide.contains(ally)) {
						// only gain reputation with factions whose fleets actually survived
						//Global.getSector().adjustPlayerReputation(new RepActionEnvelope(action, null, dialog.getTextPanel()), factionId);
					}
				}
			//}
			return true;
		}
		
		return false;
	}
	
	
	
	public void applyAfterBattleEffectsIfThereWasABattle() {
		if (!hasWinnerAndLoser() || !engagedInHostilities) {
			return;
		}
		
		gainXP();
		
//		CampaignFleetAPI winner = getWinnerData().getFleet();
//		CampaignFleetAPI loser = getLoserData().getFleet();
		List<CampaignFleetAPI> winners = battle.getSideFor(getWinnerData().getFleet());
		List<CampaignFleetAPI> losers = battle.getSideFor(getLoserData().getFleet());
		
		for (CampaignFleetAPI loser : losers) {
			if (loser.isPlayerFleet()) loser.setNoEngaging(3f);
			loser.getVelocity().set(0, 0);
		}
		for (CampaignFleetAPI winner : winners) {
			if (winner.isPlayerFleet()) winner.setNoEngaging(3f);
			winner.getVelocity().set(0, 0);
		}
		
		battle.setPlayerInvolvementFraction(computePlayerContribFraction());
		
		Global.getSector().reportBattleOccurred(battle.getPrimary(winners), battle);
		Global.getSector().reportBattleFinished(battle.getPrimary(winners), battle);
		
		CampaignFleetAPI largestWinner = battle.getPrimary(winners);
		for (CampaignFleetAPI loser : losers) {
			if (loser.getFleetData().getMembersListCopy().isEmpty()) {
				//Global.getSector().reportFleetDewspawned(loser, FleetDespawnReason.DESTROYED_BY_FLEET, winner);
				loser.despawn(FleetDespawnReason.DESTROYED_BY_BATTLE, battle);
			}
		}
		
		for (CampaignFleetAPI winner : winners) {
			if (winner.getFleetData().getMembersListCopy().isEmpty()) {
				//Global.getSector().reportFleetDewspawned(loser, FleetDespawnReason.DESTROYED_BY_FLEET, winner);
				winner.despawn(FleetDespawnReason.DESTROYED_BY_BATTLE, battle);
			}
		}
		
		for (CampaignFleetAPI enemy : battle.getBothSides()) {
			if (enemy.getAI() instanceof ModularFleetAIAPI) {
				ModularFleetAIAPI mAI = (ModularFleetAIAPI) enemy.getAI();
				mAI.getTacticalModule().forceTargetReEval();
			}
		}

		/*
		//if (loser == Global.getSector().getPlayerFleet() || winner == Global.getSector().getPlayerFleet()) {
		if (battle.isPlayerInvolved()) {
			boolean playerWon = didPlayerWinEncounter();
			List<CampaignFleetAPI> playerSide, enemySide;
			if (playerWon) {
				playerSide = winners;
				enemySide = losers;
			} else {
				playerSide = losers;
				enemySide = winners;
			}
			
			CampaignFleetAPI pf = Global.getSector().getPlayerFleet();
			pf.setMoveDestination(pf.getLocation().x, pf.getLocation().y);
			
			//boolean playerDestroyedShips =  
			// cases to cover: 1) player destroyed ships 2) player harried/harassed 3) player pursued
			// i.e. anything other than a non-destructive retreat, w/o an engagement
			
			boolean playerWasAggressive = playerDidSeriousDamage || !playerOnlyRetreated;
			RepActions action = null;
			if (engagedInHostilities) {
				action = RepActions.COMBAT_NO_DAMAGE_ESCAPE;
			}
			boolean knowsWhoPlayerIs = battle.knowsWhoPlayerIs(enemySide);
			if (playerPursued && playerWon) {
				if (knowsWhoPlayerIs) {
					action = RepActions.COMBAT_AGGRESSIVE;
				} else {
					action = RepActions.COMBAT_AGGRESSIVE_TOFF;
				}
			} else if (playerWasAggressive) {
				if (knowsWhoPlayerIs) {
					action = RepActions.COMBAT_NORMAL;
				} else {
					action = RepActions.COMBAT_NORMAL_TOFF;
				}
			}
			Set<String> seen = new HashSet<String>();
			if (action != null) {
				// use snapshot: ensure loss of reputation with factions if their fleets were destroyed
				for (CampaignFleetAPI enemy : battle.getSnapshotFor(enemySide)) {
					if (enemy.getAI() instanceof ModularFleetAIAPI) {
						ModularFleetAIAPI mAI = (ModularFleetAIAPI) enemy.getAI();
						mAI.getTacticalModule().forceTargetReEval();
					}
					
					String factionId = enemy.getFaction().getId();
					if (seen.contains(factionId)) continue;
					seen.add(factionId);
					Global.getSector().adjustPlayerReputation(action, factionId);
				}
			}
			
			if (playerWon) {
				action = RepActions.COMBAT_HELP_MINOR;
				float playerFP = 0;
				float allyFP = 0;
				float enemyFP = 0;
				for (CampaignFleetAPI fleet : battle.getSnapshotFor(winners)) {
					for (FleetMemberAPI member : fleet.getFleetData().getSnapshot()) {
						if (fleet.isPlayerFleet()) {
							playerFP += member.getFleetPointCost();
						} else {
							allyFP += member.getFleetPointCost();
						}
					}
				}
				for (CampaignFleetAPI fleet : battle.getSnapshotFor(losers)) {
					for (FleetMemberAPI member : fleet.getFleetData().getSnapshot()) {
						enemyFP += member.getFleetPointCost();
					}
				}
				if (allyFP > enemyFP) {
					action = RepActions.COMBAT_HELP_MINOR;
				} else if (allyFP < enemyFP * 0.5f) {
					action = RepActions.COMBAT_HELP_CRITICAL;
				} else {
					action = RepActions.COMBAT_HELP_MAJOR;
				}
				
				
				seen.clear();
				// non-snapshot: only gain reputation with factions whose fleets actually survived
				for (CampaignFleetAPI ally : playerSide) {
					if (ally.isPlayerFleet()) continue;
					
					if (ally.getAI() instanceof ModularFleetAIAPI) {
						ModularFleetAIAPI mAI = (ModularFleetAIAPI) ally.getAI();
						mAI.getTacticalModule().forceTargetReEval();
					}
					String factionId = ally.getFaction().getId();
					if (seen.contains(factionId)) continue;
					seen.add(factionId);
					Global.getSector().adjustPlayerReputation(action, factionId);
				}
			}
		}
		*/
	}
	
	
	
	
	public float performPostVictoryRecovery(EngagementResultAPI result) {
		EngagementResultForFleetAPI winnerResult = result.getWinnerResult();
		EngagementResultForFleetAPI loserResult = result.getLoserResult();
		return performPostVictoryRecovery(winnerResult, loserResult);
	}
	public float performPostEngagementRecoveryBoth(EngagementResultAPI result) {
		EngagementResultForFleetAPI winnerResult = result.getWinnerResult();
		EngagementResultForFleetAPI loserResult = result.getLoserResult();
		float f = performPostVictoryRecovery(winnerResult, loserResult);
		f += performPostVictoryRecovery(loserResult, winnerResult);
		f /= 2f;
		return f;
	}
		
		
	public float performPostVictoryRecovery(EngagementResultForFleetAPI winnerResult, EngagementResultForFleetAPI loserResult) {
		DataForEncounterSide winnerData = getDataFor(winnerResult.getFleet());
		DataForEncounterSide loserData = getDataFor(loserResult.getFleet());
		
		//float totalFpUsed = 0f;
		float loserDepDestroyed = 0f;
		
		for (FleetMemberAPI member : loserData.getDestroyedInLastEngagement()) {
			loserDepDestroyed += member.getDeploymentPointsCost();
		}
		for (FleetMemberAPI member : loserData.getDisabledInLastEngagement()) {
			loserDepDestroyed += member.getDeploymentPointsCost();
		}
		for (FleetMemberAPI member : loserData.getRetreatedFromLastEngagement()) {
			if (member.isFighterWing()) {
				DeployedFleetMemberAPI dfm = getDataFor(loserData.getFleet()).getMemberToDeployedMap().get(member);
				if (dfm != null && dfm.getMember() == member) {
					float deploymentCR = dfm.getShip().getWingCRAtDeployment();
					float finalCR = dfm.getShip().getRemainingWingCR();
					if (deploymentCR > finalCR) {
						float crPer = dfm.getMember().getStats().getCRPerDeploymentPercent().computeEffective(dfm.getMember().getVariant().getHullSpec().getCRToDeploy()) / 100f;
						float extraCraftLost = (deploymentCR - finalCR) / crPer;
						float wingSize = dfm.getMember().getNumFightersInWing();
						if (extraCraftLost >= 1) {
							loserDepDestroyed += Math.min(1f, extraCraftLost / wingSize) * member.getDeploymentPointsCost();
						}
					}
				}
			}
		}

		float totalRecovery = 0f;
		float count = 0f;
		for (FleetMemberAPI member : winnerData.getDeployedInLastEngagement()) {
			float dp = member.getDeploymentPointsCost();
			float recoveryFraction = Math.max(0, (dp * 0.75f - loserDepDestroyed)) / dp;
			if (loserDepDestroyed <= 0) recoveryFraction = 1f;
			
			float deployCost = getDeployCost(member);
			if (preEngagementCRForWinner.containsKey(member)) {
				float prevCR = preEngagementCRForWinner.get(member);
				if (prevCR < deployCost) {
					prevCR = deployCost;
				}
			}
			
			float recoveryAmount = Math.round(deployCost * recoveryFraction * 100f) / 100f;
			
			totalRecovery += recoveryAmount;
			count++;
			
			if (recoveryAmount <= 0) continue;
			
			
			member.getRepairTracker().applyCREvent(recoveryAmount, "Post engagement recovery");
		}

		if (count <= 0) return 0;
		
		return Math.round(totalRecovery / count * 100f) / 100f;
	}
	
	
	
	public void applyPursuitOption(CampaignFleetAPI pursuingFleet, CampaignFleetAPI otherFleet, PursuitOption pursuitOption) {
		
		if (Misc.isPlayerOrCombinedPlayerPrimary(pursuingFleet) && pursuitOption != PursuitOption.LET_THEM_GO) {
			playerOnlyRetreated = false;
		}
		
		DataForEncounterSide pursuer = getDataFor(pursuingFleet);
		DataForEncounterSide other = getDataFor(otherFleet);
		
		if (pursuitOption == PursuitOption.HARRY) {
			for (FleetMemberAPI member : otherFleet.getFleetData().getMembersListCopy()) {
				float deployCost = getDeployCost(member);
				
				float harryCost = deployCost * 1f;
				member.getRepairTracker().applyCREvent(-harryCost, "harried while disengaging");
			}
		}
	}	
	
	
	
	protected EngagementOutcome lastOutcome = null;
	public EngagementOutcome getLastEngagementOutcome() {
		return lastOutcome;
	}
	
	public boolean isBattleOver() {
		if (hasWinnerAndLoser()) return true;
		
		return lastOutcome != null &&
			   lastOutcome != EngagementOutcome.BATTLE_PLAYER_OUT_FIRST_WIN &&
			   lastOutcome != EngagementOutcome.BATTLE_PLAYER_OUT_FIRST_LOSS &&
			   lastOutcome != EngagementOutcome.BATTLE_ENEMY_WIN &&
			   lastOutcome != EngagementOutcome.BATTLE_PLAYER_WIN;
	}
	
	public boolean wasLastEngagementEscape() {
		return lastOutcome != null &&
//			   lastOutcome != EngagementOutcome.ESCAPE_PLAYER_OUT_FIRST_WIN &&
//			   lastOutcome != EngagementOutcome.ESCAPE_PLAYER_OUT_FIRST_LOSS &
		   	   lastOutcome != EngagementOutcome.BATTLE_ENEMY_WIN &&
		       lastOutcome != EngagementOutcome.BATTLE_PLAYER_WIN;
	}
	
	public boolean didPlayerWinLastEngagement() {
		return lastOutcome == EngagementOutcome.BATTLE_PLAYER_WIN ||
		   	   lastOutcome == EngagementOutcome.BATTLE_PLAYER_WIN_TOTAL ||
		   	   lastOutcome == EngagementOutcome.BATTLE_PLAYER_OUT_FIRST_WIN ||
		   	   lastOutcome == EngagementOutcome.PURSUIT_PLAYER_OUT_FIRST_WIN ||
		   	   lastOutcome == EngagementOutcome.ESCAPE_PLAYER_OUT_FIRST_WIN ||
		   	   lastOutcome == EngagementOutcome.ESCAPE_ENEMY_LOSS_TOTAL ||
		   	   lastOutcome == EngagementOutcome.ESCAPE_ENEMY_SUCCESS ||
		   	   lastOutcome == EngagementOutcome.ESCAPE_PLAYER_WIN ||
		   	   lastOutcome == EngagementOutcome.ESCAPE_PLAYER_WIN_TOTAL;
	}
	
	public boolean didPlayerWinEncounter() {
		if (getDataFor(Global.getSector().getPlayerFleet()).disengaged()) return false;
		
		// non-fighting "win", i.e. harrying a weaker enemy
		//if (lastOutcome == null && getWinner() == Global.getSector().getPlayerFleet()) {
		//if (lastOutcome == null && battle.isPlayerSide(battle.getSideFor(getWinner()))) { 
		if ((lastOutcome == null || hasWinnerAndLoser()) && battle.isPlayerSide(battle.getSideFor(getWinner()))) { 
			return true;
		}
		
		return lastOutcome == EngagementOutcome.BATTLE_PLAYER_WIN_TOTAL ||
			   //lastOutcome == EngagementOutcome.BATTLE_PLAYER_WIN ||
		   	   lastOutcome == EngagementOutcome.ESCAPE_ENEMY_LOSS_TOTAL ||
		   	   lastOutcome == EngagementOutcome.ESCAPE_ENEMY_SUCCESS ||
		   	   lastOutcome == EngagementOutcome.ESCAPE_PLAYER_WIN ||
		   	   lastOutcome == EngagementOutcome.ESCAPE_PLAYER_WIN_TOTAL;		
	}
	
	
	public int getCreditsLooted() {
		return creditsLooted;
	}
	
	public float getSalvageMult(Status status) {
		float mult = 1f;
		switch (status) {
		case DESTROYED:
			mult = 0.5f;
			break;
		case DISABLED:
			mult = 1f;
			break;
		case REPAIRED:
			mult = 1f;
			break;
		case CAPTURED:
			mult = 0.25f;
			break;
		}
		return mult;
	}
	
	public List<FleetMemberAPI> repairShips() {
		DataForEncounterSide winner = getWinnerData();
		return repairShips(winner);
		
//		DataForEncounterSide loser = getLoserData();
//		repairShips(loser);
	}
	
	public static enum EngageBoardableOutcome {
		ESCAPED,
		DISABLED,
		DESTROYED,
	}
	
	public EngageBoardableOutcome engageBoardableShip(FleetMemberAPI toBoard, 
								CampaignFleetAPI fleetItBelongsTo,
								CampaignFleetAPI attackingFleet) {
		float r = (float) Math.random();
//		if (r < ENGAGE_ESCAPE_CHANCE && !Misc.isPlayerOrCombinedContainingPlayer(attackingFleet)) {
//			// escaped
//			CampaignFleetAPI fleet = getBattle().getSourceFleet(toBoard);
//			letBoardableGo(toBoard, fleet, attackingFleet);
//			
//			return EngageBoardableOutcome.ESCAPED;
//		} else
		if (r < ENGAGE_ESCAPE_CHANCE + ENGAGE_DISABLE_CHANCE) {
			// disabled
			DataForEncounterSide attackerSide = getDataFor(attackingFleet);
			attackerSide.changeEnemy(toBoard, Status.DISABLED);
			toBoard.getStatus().disable();
			return EngageBoardableOutcome.DISABLED;
		} else {
			DataForEncounterSide attackerSide = getDataFor(attackingFleet);
			attackerSide.changeEnemy(toBoard, Status.DESTROYED);
			toBoard.getStatus().disable();
			return EngageBoardableOutcome.DESTROYED;
		}
	}
	
	
	public static enum BoardingAttackType {
		SHIP_TO_SHIP,
		LAUNCH_FROM_DISTANCE,
	}
	public static enum BoardingOutcome {
		SUCCESS,
		SELF_DESTRUCT,
		SUCCESS_TOO_DAMAGED,
		SHIP_ESCAPED,
		SHIP_ESCAPED_CLEAN,
	}
	
	public static class BoardingResult {
		private BoardingOutcome outcome;
		private CrewCompositionAPI attackerLosses = Global.getFactory().createCrewComposition();
		private CrewCompositionAPI defenderLosses = Global.getFactory().createCrewComposition();
		private FleetMemberAPI member;
		private List<FleetMemberAPI> lostInSelfDestruct = new ArrayList<FleetMemberAPI>();
		
		public BoardingOutcome getOutcome() {
			return outcome;
		}
		public List<FleetMemberAPI> getLostInSelfDestruct() {
			return lostInSelfDestruct;
		}
		public void setOutcome(BoardingOutcome outcome) {
			this.outcome = outcome;
		}
		public CrewCompositionAPI getAttackerLosses() {
			return attackerLosses;
		}
		public void setAttackerLosses(CrewCompositionAPI attackerLosses) {
			this.attackerLosses = attackerLosses;
		}
		public CrewCompositionAPI getDefenderLosses() {
			return defenderLosses;
		}
		public void setDefenderLosses(CrewCompositionAPI defenderLosses) {
			this.defenderLosses = defenderLosses;
		}
		public FleetMemberAPI getMember() {
			return member;
		}
		public void setMember(FleetMemberAPI member) {
			this.member = member;
		}
		
	}
	
	public static final float SELF_DESTRUCT_CHANCE = 0.25f;
	public static final float CIV_SELF_DESTRUCT_CHANCE = 0.05f;
	
	public static final float ENGAGE_ESCAPE_CHANCE = 0.25f;
	public static final float ENGAGE_DISABLE_CHANCE = 0.5f;
	public static final float ENGAGE_DESTROY_CHANCE = 0.25f;
	
	public static final float LAUNCH_CLEAN_ESCAPE_CHANCE = 0.5f;
	public static final float DOCK_SUCCESS_CHANCE = 0.5f;
	public static final float LAUNCH_SUCCESS_CHANCE = 0.25f;
	
	//public static final float DEFENDER_BONUS = 4f;
	//public static final float DEFENDER_VS_LAUNCH_BONUS = 3f;
	
	public BoardingResult boardShip(FleetMemberAPI member, CampaignFleetAPI attacker, CampaignFleetAPI defender) {
		
		
		DataForEncounterSide attackerSide = getDataFor(attacker);
		DataForEncounterSide defenderSide = getDataFor(defender);
		
		float attackerMarineMult = attacker.getCommanderStats().getMarineEffectivnessMult().getModifiedValue();
		float defenderMarineMult = defender.getCommanderStats().getMarineEffectivnessMult().getModifiedValue();
		
		float greenMult = 1f;
		float regularMult = 1.25f;
		float veteranMult = 1.5f;
		float eliteMult = 2f;
		float marineMult = 7f;
		
		
		float attackerStr = attacker.getCargo().getMarines() * marineMult;
		attackerStr *= attackerMarineMult;
		
		CrewCompositionAPI defenderCrew = member.getCrewComposition();
		float defenderStr = defenderCrew.getGreen() * greenMult + 
							defenderCrew.getRegular() * regularMult +
							defenderCrew.getVeteran() * veteranMult +
							defenderCrew.getElite() * eliteMult +
							defenderCrew.getMarines() * marineMult;
		defenderStr *= defenderMarineMult;

		defenderStr *= Global.getSettings().getFloat("boardingDifficulty");
		
		Random rand = new Random(1300000 * (member.getId().hashCode() + defender.getId().hashCode() + Global.getSector().getClock().getDay()));
		attackerStr *= 0.75f + 0.25f * rand.nextFloat();
		defenderStr *= 0.75f + 0.25f * rand.nextFloat();
		
		boolean attackerWin = attackerStr > defenderStr;
		boolean defenderWin = !attackerWin;
		
		BoardingResult result = new BoardingResult();
		result.setMember(member);
		
		
		BoardingOutcome outcome = BoardingOutcome.SUCCESS;
		if (defenderWin) {
			outcome = BoardingOutcome.SHIP_ESCAPED;
		}
		
		CrewCompositionAPI boardingParty = Global.getFactory().createCrewComposition();
		boardingParty.addMarines(attacker.getCargo().getMarines());
		
		result.setOutcome(outcome);
		switch (outcome) {
		case SHIP_ESCAPED:
			computeCrewLossFromBoarding(result, member, boardingParty, attackerStr, defenderStr);
			result.getAttackerLosses().removeFromCargo(attacker.getCargo());
			member.getCrewComposition().removeAll(result.getDefenderLosses());
			
			letBoardableGo(member, defender, attacker);
			break;
		case SUCCESS:
			computeCrewLossFromBoarding(result, member, boardingParty, attackerStr, defenderStr);
			result.getAttackerLosses().removeFromCargo(attacker.getCargo());
			member.getCrewComposition().removeAll(result.getDefenderLosses());
			
			//attackerSide.removeEnemyCasualty(member);
			attacker.getFleetData().addFleetMember(member);
			getBattle().getCombinedFor(attacker).getFleetData().addFleetMember(member);
			
			member.getRepairTracker().setMothballed(true);
			//defender.getFleetData().removeFleetMember(member);
			
			attackerSide.changeEnemy(member, Status.CAPTURED);
			defenderSide.changeOwn(member, Status.CAPTURED);
			
			attackerSide.getInReserveDuringLastEngagement().add(member);
			defenderSide.getDestroyedInLastEngagement().remove(member);
			defenderSide.getDisabledInLastEngagement().remove(member);
			
			member.setOwner(0);	
			break;
		}

		return result;
	}
	
	public float getBoardingSuccessPercent(FleetMemberAPI member, CampaignFleetAPI attacker, CampaignFleetAPI defender) {
		DataForEncounterSide attackerSide = getDataFor(attacker);
		DataForEncounterSide defenderSide = getDataFor(defender);
		float attackerMarineMult = attacker.getCommanderStats().getMarineEffectivnessMult().getModifiedValue();
		float defenderMarineMult = defender.getCommanderStats().getMarineEffectivnessMult().getModifiedValue();
		
		float greenMult = 1f;
		float regularMult = 1.25f;
		float veteranMult = 1.5f;
		float eliteMult = 2f;
		float marineMult = 7f;
		
		Random rand = new Random();
		
		float wins = 0;
		float losses = 0;
		
		for (int i = 0; i < 100; i++) {
			float attackerStr = attacker.getCargo().getMarines() * marineMult;
			attackerStr *= attackerMarineMult;
			
			CrewCompositionAPI defenderCrew = member.getCrewComposition();
			float defenderStr = defenderCrew.getGreen() * greenMult + 
								defenderCrew.getRegular() * regularMult +
								defenderCrew.getVeteran() * veteranMult +
								defenderCrew.getElite() * eliteMult +
								defenderCrew.getMarines() * marineMult;
			defenderStr *= defenderMarineMult;
	
			defenderStr *= Global.getSettings().getFloat("boardingDifficulty");
			
			attackerStr *= 0.75f + 0.25f * rand.nextFloat();
			defenderStr *= 0.75f + 0.25f * rand.nextFloat();
		
			boolean attackerWin = attackerStr > defenderStr;
			if (attackerWin) wins++;
			else losses++;
		}
		
		return wins;
	}
	
	
	protected void computeMissedLaunchLosses(BoardingResult result, CrewCompositionAPI boardingParty) {
		result.getAttackerLosses().addAll(boardingParty);
		result.getAttackerLosses().multiplyBy((float) Math.random() * 0.2f);
	}
	
	protected void computeCrewLossFromBoarding(BoardingResult result,
							FleetMemberAPI member, CrewCompositionAPI boardingParty,
							float attackerStr, float defenderStr) {
		
		if (attackerStr < 1) attackerStr = 1;
		if (defenderStr < 1) defenderStr = 1;
		float cap = 2f;
		if (attackerStr > defenderStr * cap) {
			attackerStr = defenderStr * cap;
		}
		if (defenderStr > attackerStr * cap) {
			defenderStr = attackerStr * cap;
		}
		
		float attackerLosses = defenderStr / (attackerStr + defenderStr);
		float defenderLosses = attackerStr / (attackerStr + defenderStr);
		
		if (attackerStr > defenderStr) {
			result.getAttackerLosses().addAll(boardingParty);
			result.getAttackerLosses().multiplyBy(attackerLosses);
			result.getDefenderLosses().addAll(member.getCrewComposition());
			result.getDefenderLosses().multiplyBy(defenderLosses);
		} else {
			result.getAttackerLosses().addAll(boardingParty);
			result.getAttackerLosses().multiplyBy(attackerLosses);
			result.getDefenderLosses().addAll(member.getCrewComposition());
			result.getDefenderLosses().multiplyBy(defenderLosses);
		}
		//member.getCrewComposition().removeAll(result.getDefenderLosses());
	}
	
	
	protected void applyBoardingSelfDestruct(FleetMemberAPI member, 
			 CrewCompositionAPI boardingParty, BoardingAttackType attackType,
			 List<FleetMemberAPI> boardingTaskForce,
			 CampaignFleetAPI attacker, CampaignFleetAPI defender,
			 BoardingResult result) {
		
		DataForEncounterSide attackerSide = getDataFor(attacker);
		DataForEncounterSide defenderSide = getDataFor(defender);
		
		attackerSide.changeEnemy(member, Status.DESTROYED);
		defenderSide.changeOwn(member, Status.DESTROYED);

		
		CrewCompositionAPI total = Global.getFactory().createCrewComposition();
		
		if (attackType == BoardingAttackType.SHIP_TO_SHIP) {
			for (FleetMemberAPI fm : boardingTaskForce) {
				float damage = member.getStats().getFluxCapacity().getModifiedValue() * (1f + (float) Math.random() * 0.5f);
				float hull = fm.getStatus().getHullFraction();
				float hullDamageFactor = 0f;
				fm.getStatus().applyDamage(damage);
				if (fm.getStatus().getHullFraction() <= 0) {
					fm.getStatus().disable();
					attacker.getFleetData().removeFleetMember(fm);
					attackerSide.addOwn(fm, Status.DESTROYED);
					//total.addAll(fm.getCrewComposition());
					
					attackerSide.getRetreatedFromLastEngagement().remove(fm);
					attackerSide.getInReserveDuringLastEngagement().remove(fm);
					attackerSide.getDeployedInLastEngagement().remove(fm);
					attackerSide.getDestroyedInLastEngagement().add(fm);
					
					result.getLostInSelfDestruct().add(fm);
					
					hullDamageFactor = 1f;
				} else {
					float newHull = fm.getStatus().getHullFraction();
					float diff = hull - newHull;
					if (diff < 0) diff = 0;
					hullDamageFactor = diff;
				}
				CrewCompositionAPI temp = Global.getFactory().createCrewComposition();
				temp.addAll(fm.getCrewComposition());
				float lossFraction = computeLossFraction(fm, null, fm.getStatus().getHullFraction(), hullDamageFactor);
				temp.multiplyBy(lossFraction);
				total.addAll(temp);
			}
			//total.removeAll(boardingParty);
		}
		
		float lossFraction = computeLossFraction(null, null, 0f, 1f);
		total.setMarines(Math.round(Math.max(total.getMarines(), boardingParty.getMarines() * lossFraction)));
		total.setGreen(Math.round(Math.max(total.getGreen(), boardingParty.getGreen() * lossFraction)));
		total.setRegular(Math.round(Math.max(total.getRegular(), boardingParty.getRegular() * lossFraction)));
		total.setVeteran(Math.round(Math.max(total.getVeteran(), boardingParty.getVeteran() * lossFraction)));
		total.setElite(Math.round(Math.max(total.getElite(), boardingParty.getElite() * lossFraction)));
		
		//result.getAttackerLosses().addAll(boardingParty);
//		float lossFraction = computeLossFraction(boardingTaskForce.get(0), 0f, 1f);
//		total.multiplyBy(lossFraction);
		
		result.getAttackerLosses().addAll(total);
		result.getDefenderLosses().addAll(member.getCrewComposition());
	}
	
	public void letBoardableGo(FleetMemberAPI toBoard, CampaignFleetAPI fleetItBelongsTo, CampaignFleetAPI attackingFleet) {
		DataForEncounterSide attackerSide = getDataFor(attackingFleet);
		attackerSide.removeEnemyCasualty(toBoard);
		
		DataForEncounterSide defenderSide = getDataFor(fleetItBelongsTo);
		defenderSide.removeOwnCasualty(toBoard);
		
		
		defenderSide.getDestroyedInLastEngagement().remove(toBoard);
		defenderSide.getDisabledInLastEngagement().remove(toBoard);
		defenderSide.getRetreatedFromLastEngagement().add(toBoard);
		
		if (!fleetItBelongsTo.isValidPlayerFleet()) {
			fleetItBelongsTo.getCargo().removeCrew(CrewXPLevel.GREEN, fleetItBelongsTo.getCargo().getCrew(CrewXPLevel.GREEN));
			fleetItBelongsTo.getCargo().removeCrew(CrewXPLevel.REGULAR, fleetItBelongsTo.getCargo().getCrew(CrewXPLevel.REGULAR));
			fleetItBelongsTo.getCargo().removeCrew(CrewXPLevel.VETERAN, fleetItBelongsTo.getCargo().getCrew(CrewXPLevel.VETERAN));
			fleetItBelongsTo.getCargo().removeCrew(CrewXPLevel.ELITE, fleetItBelongsTo.getCargo().getCrew(CrewXPLevel.ELITE));
			fleetItBelongsTo.getCargo().removeMarines(fleetItBelongsTo.getCargo().getMarines());
		}
		
		FleetDataAPI data = fleetItBelongsTo.getFleetData();
		data.addFleetMember(toBoard);
		
		getBattle().getCombinedFor(fleetItBelongsTo).getFleetData().addFleetMember(toBoard);
		
		toBoard.getCrewComposition().addToCargo(fleetItBelongsTo.getCargo());
	}
	
	public CrewCompositionAPI getMaxBoarders(CampaignFleetAPI fleet, List<FleetMemberAPI> boardingTaskForce) {
		CrewCompositionAPI c = Global.getFactory().createCrewComposition();
		float totalCapacity = 0;
		for (FleetMemberAPI member : boardingTaskForce) {
			c.addMarines(Math.max(0, member.getMaxCrew() - member.getMinCrew()));
			c.addGreen(member.getCrewComposition().getGreen());
			c.addRegular(member.getCrewComposition().getRegular());
			c.addVeteran(member.getCrewComposition().getVeteran());
			c.addElite(member.getCrewComposition().getElite());
			totalCapacity += member.getMaxCrew();
		}
		c.setMarines(Math.min(fleet.getCargo().getMarines(), c.getMarines()));
		
		CrewCompositionAPI temp = Global.getFactory().createCrewComposition();
		temp.addGreen(fleet.getCargo().getCrew(CrewXPLevel.GREEN));
		temp.addRegular(fleet.getCargo().getCrew(CrewXPLevel.REGULAR));
		temp.addVeteran(fleet.getCargo().getCrew(CrewXPLevel.VETERAN));
		temp.addElite(fleet.getCargo().getCrew(CrewXPLevel.ELITE));
		temp.removeAll(c);
		
		for (CrewXPLevel level : EnumSet.allOf(CrewXPLevel.class)) {
			float diff = totalCapacity - (c.getTotalCrew() + c.getMarines());
			if (diff > 0) {
				float crew = temp.getCrew(level);
				float add = Math.min(diff, crew);
				if (add > 0) {
					c.addCrew(level, add);
				}
			} else {
				break;
			}
		}
		return c;
	}
	
	
	public FleetMemberAPI pickShipToBoard(CampaignFleetAPI winningFleet, CampaignFleetAPI otherFleet) {
		//if ((float) Math.random() > 0.5f) return null;
		//if (true) return null;
		
		
		if (Misc.isPlayerOrCombinedContainingPlayer(winningFleet) && 
				Global.getSector().getPlayerFleet().getFleetData().getMembersListCopy().size() >= Global.getSettings().getMaxShipsInFleet()) {
			return null;
		}
		
		if (Misc.isPlayerOrCombinedContainingPlayer(otherFleet)) {
			return null;
		}
		
		DataForEncounterSide winnerData = getDataFor(winningFleet);
		DataForEncounterSide loserData = getDataFor(otherFleet);
		
		
		WeightedRandomPicker<FleetMemberAPI> candidates = new WeightedRandomPicker<FleetMemberAPI>();
		float crewCapacityLost = 0f;
		float fpLost = 0f;
		for (FleetMemberData data : winnerData.getEnemyCasualties()) {
			crewCapacityLost += data.getMember().getMaxCrew();
			
			if (data.getMember().isFighterWing()) continue;
			if (data.getStatus() != Status.DISABLED) continue;
			
			fpLost += data.getMember().getFleetPointCost();

			float boardingChance = Global.getSettings().getFloat("boardingChance");
			boardingChance *= data.getMember().getStats().getDynamic().getValue(Stats.BOARDING_CHANCE_MULT);
			Random rand = new Random(1000000 * (data.getMember().getId().hashCode() + otherFleet.getId().hashCode() + Global.getSector().getClock().getDay()));
			if (rand.nextFloat() < boardingChance) {
			//if (data.getMember().getHullId().equals("enforcer_d_pirates")) {
			//if (data.getMember().getHullId().contains("lasher")) {
			//if (rand.nextFloat() < boardingChance || true) {
				candidates.add(data.getMember(), 1f);
			}
		}
		if (fpLost < 1) fpLost = 1;
		
		if (candidates.isEmpty()) return null;
		
		float crewCapacityRemaining = 0f;
		for (FleetMemberAPI member : otherFleet.getFleetData().getMembersListCopy()) {
			crewCapacityRemaining += member.getMaxCrew();
		}
		if (crewCapacityRemaining < 1) crewCapacityRemaining = 1;
		
		FleetMemberAPI pick = candidates.pick();
		CrewCompositionAPI crew = Global.getFactory().createCrewComposition();

		float f = computeRecoverableFraction(pick, null, 0f, 1f);

		float marineFactor = crewCapacityLost / (crewCapacityLost + crewCapacityRemaining);
		marineFactor *= f * (float) pick.getFleetPointCost() / fpLost;
		if (marineFactor > 1) marineFactor = 1;
		
		float marinesOnBoard = (int) (otherFleet.getCargo().getMarines() * marineFactor);
		crew.setMarines(marinesOnBoard);
		
		float maxCrew = pick.getMinCrew();
		float totalCrew = pick.getCrewComposition().getTotalCrew();
		if (totalCrew <= 1) totalCrew = 1;
		for (CrewXPLevel level : EnumSet.allOf(CrewXPLevel.class)) {
			float count = pick.getCrewComposition().getCrew(level);
			count *= maxCrew / totalCrew * (0.7f + (float) Math.random() * 0.2f);
			pick.getCrewComposition().setCrew(level, (int) count);
		}
		crew.addAll(pick.getCrewComposition());
		
		repairFleetMember(pick);
		
		//ShipToBoard result = new ShipToBoard(pick, crew);
		pick.getCrewComposition().removeAllCrew();
		pick.getCrewComposition().addAll(crew);
		
		if (pick.getCrewComposition().getTotalCrew() < 2) {
			float count = maxCrew * (0.7f + (float) Math.random() * 0.2f);
			pick.getCrewComposition().setCrew(pick.getCrewXPLevel(), (int) count);
			//pick.getCrewComposition().addCrew(pick.getCrewXPLevel(), 2);
		}
		return pick;
	}
	
	
	protected List<FleetMemberAPI> repairShips(DataForEncounterSide side) {
		if (side == null || side.disengaged()) return new ArrayList<FleetMemberAPI>();
		//if (lostBattle) return;
		
		List<FleetMemberAPI> result = new ArrayList<FleetMemberAPI>();
		
		for (FleetMemberData data : side.getOwnCasualties()) {
			FleetMemberAPI member = data.getMember();
			if (member.isFighterWing()) continue;
			if (member.isAlly()) continue;
			
			//float threshold = 1f - getRepairChance(member, side.getFleet());
			float threshold = 1f - getRepairChance(member, battle.getSourceFleet(member));
			//threshold = 0f;
			
			if ((float) Math.random() > threshold) {
				repairFleetMember(member);
				
				data.setStatus(Status.REPAIRED);
				side.getFleet().getFleetData().addFleetMember(member);
				side.getInReserveDuringLastEngagement().add(member);
				battle.getSourceFleet(member).getFleetData().addFleetMember(member);
				
				result.add(member);
			}
		}
		
		for (FleetMemberAPI member : result) {
			side.removeOwnCasualty(member);
		}

		return result;
	}
	
	protected void repairFleetMember(FleetMemberAPI member) {
		member.getStatus().repairDisabledABit();
		
		ShipVariantAPI variant = member.getVariant().clone();
		variant.clearHullMods();
		variant.setNumFluxCapacitors(0);
		variant.setNumFluxVents(0);
		variant.setSource(VariantSource.REFIT);
		member.setVariant(variant, false, false);
		List<String> remove = new ArrayList<String>();
		if (!member.isFighterWing()) {
			for (String slotId : variant.getNonBuiltInWeaponSlots()) {
				if ((float) Math.random() > getWeaponSurvivialOnRepairChance()) {
					remove.add(slotId);
				}
			}
			for (String slotId : remove) {
				variant.clearSlot(slotId);
			}
		}
	}
	
	
	public void gainXP() {
		if (sideData.size() != 2) return;
		if (!battle.isPlayerInvolved()) return;

		DataForEncounterSide sideOne = sideData.get(0);
		DataForEncounterSide sideTwo = sideData.get(1);
		if (battle.isPlayerSide(battle.getSideFor(sideOne.getFleet()))) {
			gainXP(sideOne, sideTwo);
		} else if (battle.isPlayerSide(battle.getSideFor(sideTwo.getFleet()))) {
			gainXP(sideTwo, sideOne);
		}
	}
	
	protected void gainOfficerXP(DataForEncounterSide data, float xp) {
		float max = data.getMaxTimeDeployed();
		if (max < 1) max = 1;
		float num = data.getOfficerData().size();
		if (num < 1) num = 1;
		for (PersonAPI person : data.getOfficerData().keySet()) {
			OfficerEngagementData oed = data.getOfficerData().get(person);
			if (oed.sourceFleet == null || !oed.sourceFleet.isPlayerFleet()) continue;
			
			OfficerDataAPI od = oed.sourceFleet.getFleetData().getOfficerData(person);
			if (od == null) continue; // shouldn't happen, as this is checked earlier before it goes into the map
			
			float f = oed.timeDeployed / max;
			if (f < 0) f = 0;
			if (f > 1) f = 1;
			
			od.addXP((long)(f * xp / num));
		}
	}
	
	
	public float getPlayerFPHullDamageToEnemies() {
		return playerFPHullDamageToEnemies;
	}

	public void setPlayerFPHullDamageToEnemies(float playerFPHullDamageToEnemies) {
		this.playerFPHullDamageToEnemies = playerFPHullDamageToEnemies;
	}

	public float getAllyFPHullDamageToEnemies() {
		return allyFPHullDamageToEnemies;
	}

	public void setAllyFPHullDamageToEnemies(float allyFPHullDamageToEnemies) {
		this.allyFPHullDamageToEnemies = allyFPHullDamageToEnemies;
	}

	protected float playerFPHullDamageToEnemies = 0f;
	protected float allyFPHullDamageToEnemies = 0f;
	protected float playerFPHullDamageToAllies = 0f;
	protected Map<FactionAPI, Float> playerFPHullDamageToAlliesByFaction = new HashMap<FactionAPI, Float>();
	protected void computeFPHullDamage() {
		if (runningDamageTotal == null) return;
		
//		playerFPHullDamageToEnemies = 0f;
//		allyFPHullDamageToEnemies = 0f;
//		playerFPHullDamageToAllies = 0f;
		
		for (FleetMemberAPI member : runningDamageTotal.getDealt().keySet()) {
			if (member.getOwner() != 0) continue;
			
			
			DealtByFleetMember dealt = runningDamageTotal.getDealt().get(member);
			for (FleetMemberAPI target : dealt.getDamage().keySet()) {
				DamageToFleetMember damage = dealt.getDamageTo(target);
				float maxHull = target.getStats().getHullBonus().computeEffective(target.getHullSpec().getHitpoints());
				if (maxHull <= 0) continue;
				if (target.isFighterWing()) {
					maxHull *= target.getNumFightersInWing();
				}
				
				float currDam = Math.min(damage.hullDamage, maxHull) / maxHull * (float) target.getFleetPointCost();
				if (target.getOwner() == 1) {
					if (member.isAlly()) {
						allyFPHullDamageToEnemies += currDam;
					} else {
						playerFPHullDamageToEnemies += currDam;
					}
				} else if (target.isAlly()) {
					playerFPHullDamageToAllies += currDam;
					CampaignFleetAPI fleet = battle != null ? battle.getSourceFleet(target) : null;
					if (fleet != null) {
						float curr = currDam;
						if (playerFPHullDamageToAlliesByFaction.containsKey(fleet.getFaction())) {
							curr += playerFPHullDamageToAlliesByFaction.get(fleet.getFaction());
						}
						playerFPHullDamageToAlliesByFaction.put(fleet.getFaction(), curr);
					}
				}
			}
		}
//		allyFPHullDamageToEnemies += playerFPHullDamageToEnemies;
//		playerFPHullDamageToEnemies = 0f;
		runningDamageTotal = null;
	}
	
	
	protected float computePlayerContribFraction() {
		float total = playerFPHullDamageToEnemies + allyFPHullDamageToEnemies;
		if (total <= 0) {
			if (battle == null) return 1f;
			//return battle.getPlayerInvolvementFraction();
			return 0f;
		}
		
		boolean hasAllies = false;
		boolean startedWithAllies = false;
		if (battle != null) {
			hasAllies = battle.getPlayerSide().size() <= 1;
			startedWithAllies = battle.getPlayerSideSnapshot().size() > 1;
		}
		if (startedWithAllies) { // && hasAllies) {
			return Math.min(0.9f, playerFPHullDamageToEnemies / total);
		} else {
			return 1f;
		}
	}
	
	
	protected void gainXP(DataForEncounterSide side, DataForEncounterSide otherSide) {
		//CampaignFleetAPI fleet = side.getFleet();
		CampaignFleetAPI fleet = Global.getSector().getPlayerFleet();
		int fpTotal = 0;
		for (FleetMemberData data : otherSide.getOwnCasualties()) {
			float fp = data.getMember().getFleetPointCost();
			fp *= 1f + data.getMember().getCaptain().getStats().getLevel() / 5f;
			fpTotal += fp;
		}
		
		float xp = (float) fpTotal * 250;
		xp *= 2f;
		
		xp *= computePlayerContribFraction();
		
		if (xp > 0) {
			fleet.getCargo().gainCrewXP(xp);
			
			//if (side.getFleet().isPlayerFleet()) {
			//}
			// only gain XP if it's the player fleet anyway, no need to check this here
			gainOfficerXP(side, xp);
			
			fleet.getCommander().getStats().addXP((long) xp);
			fleet.getCommander().getStats().levelUpIfNeeded();
		}
	}
	
	
	
	protected CargoAPI loot = Global.getFactory().createCargo(false);
	protected int creditsLooted = 0;
	public void generateLoot() {
		creditsLooted = 0;
		loot.clear();
		//if (getWinner() == Global.getSector().getPlayerFleet()) {
		if (battle.isPlayerSide(battle.getSideFor(getWinner()))) {
			generatePlayerLoot();
		} else { //if (getLoser() == Global.getSector().getPlayerFleet()) {
			handleCargoLooting();
		}
		loot.sort();
	}
	
	protected void generatePlayerLoot() {
		//computeFPHullDamage();
		
		
		DataForEncounterSide winner = getWinnerData();
		DataForEncounterSide loser = getLoserData();

		if (winner == null || loser == null) return;
	
		float adjustedFPSalvage = 0;
		float playerContribMult = computePlayerContribFraction();
		
		for (FleetMemberData data : winner.getEnemyCasualties()) {
			if (data.getStatus() == Status.REPAIRED) {
				continue;
			}
			float mult = getSalvageMult(data.getStatus()) * playerContribMult;
			lootWeapons(data.getMember(), mult);
			adjustedFPSalvage += (float) data.getMember().getFleetPointCost() * mult;
		}
		for (FleetMemberData data : winner.getOwnCasualties()) {
			if (data.getMember().isAlly()) continue;
			
			if (data.getStatus() == Status.CAPTURED || data.getStatus() == Status.REPAIRED) {
				continue;
			}
			float mult = getSalvageMult(data.getStatus());
			lootWeapons(data.getMember(), mult);
			
			adjustedFPSalvage += (float) data.getMember().getFleetPointCost() * mult;
		}
		

		float minCreditsFraction = Global.getSettings().getFloat("salvageFractionCreditsMin");
		float maxCreditsFraction = Global.getSettings().getFloat("salvageFractionCreditsMax");
		
		float creditsFraction = minCreditsFraction + (maxCreditsFraction - minCreditsFraction) * (float) Math.random();
		creditsFraction *= playerContribMult;
		
		float maxSalvageValue = adjustedFPSalvage * Global.getSettings().getFloat("salvageValuePerFP");
		if (Misc.isEasy()) {
			 maxSalvageValue *= Global.getSettings().getFloat("easySalvageMult");
		}
		
		creditsLooted = Math.round(maxSalvageValue * creditsFraction);
		maxSalvageValue -= creditsLooted;
		
		float salvageValue = 0f;
		WeightedRandomPicker<String> lootPicker = new WeightedRandomPicker<String>();
		lootPicker.add(Commodities.METALS, 20);
		lootPicker.add(Commodities.SUPPLIES, 10);
		lootPicker.add(Commodities.FUEL, 5);
		lootPicker.add(Commodities.HEAVY_MACHINERY, 1);
		
		while (salvageValue < maxSalvageValue) {
			String commodityId = lootPicker.pick();
			if (commodityId == null) break;
			
			CommoditySpecAPI spec = Global.getSector().getEconomy().getCommoditySpec(commodityId);
			float qty = 1f;
			salvageValue += spec.getBasePrice() * qty;
			loot.addCommodity(commodityId, qty);
		}
		
		handleCargoLooting();
	}
	
	protected static class LootableCargoStack {
		public CargoAPI source;
		public CargoStackAPI stack;
		public LootableCargoStack(CargoAPI source, CargoStackAPI stack) {
			this.source = source;
			this.stack = stack;
		}
		
	}
	protected void handleCargoLooting() {
		DataForEncounterSide winner = getWinnerData();
		DataForEncounterSide loser = getLoserData();

		if (winner == null || loser == null) return;
		
		CargoAPI loserCargo = (CargoAPI) loser.getFleet().getCargo();
		float maxCargo = loserCargo.getMaxCapacity();
		float maxFuel = loserCargo.getMaxFuel();

		
		float lostCargo = 0f;
		float lostFuel = 0f;
		for (FleetMemberData data : winner.getEnemyCasualties()) {
			if (data.getStatus() == Status.REPAIRED) {
				continue;
			}
			lostCargo += data.getMember().getCargoCapacity();
			lostFuel += data.getMember().getFuelCapacity();
			
			maxCargo += data.getMember().getCargoCapacity();
			maxFuel += data.getMember().getFuelCapacity();
		}
		
		if (lostCargo <= 0 && lostFuel <= 0) {
			return;
		}
		
		if (maxCargo < 1) maxCargo = 1;
		if (maxFuel < 1) maxFuel = 1;
		
		float recoveryFraction = Global.getSettings().getFloat("savlageCargoFraction");
		
		if (battle.isPlayerSide(battle.getSideFor(winner.getFleet()))) {
			float playerContribMult = computePlayerContribFraction();
			recoveryFraction *= playerContribMult;
		}
		
		float cargoFractionLost = lostCargo / maxCargo;
		float fuelFractionLost = lostFuel / maxFuel;
		if (lostCargo > maxCargo) cargoFractionLost = 1f;
		if (lostFuel > maxFuel) fuelFractionLost = 1f;
		
		
		List<CampaignFleetAPI> losers = battle.getSideFor(loser.getFleet());
		List<LootableCargoStack> stacks = new ArrayList<LootableCargoStack>();
		for (CampaignFleetAPI curr : losers) {
			for (CargoStackAPI stack : curr.getCargo().getStacksCopy()) {
				stacks.add(new LootableCargoStack(curr.getCargo(), stack));
			}
		}
		
		for (LootableCargoStack stack : stacks) {
			if (stack.stack.isNull()) continue;
			if (stack.stack.isPersonnelStack()) continue;
			if (stack.stack.getSize() < 1) continue;
			
			float numLost = 0;
			float numTaken = 0;
			if (stack.stack.isFuelStack()) {
				numLost = fuelFractionLost * stack.stack.getSize();
				numTaken = Math.round(numLost * (0.5f + (float) Math.random() * 0.5f));
			} else {
				numLost = cargoFractionLost * stack.stack.getSize();
				numTaken = Math.round(numLost * (0.5f + (float) Math.random() * 0.5f));
			}
			
			stack.stack.add(-numLost);
			if (numTaken * recoveryFraction >= 1) {
				loot.addItems(stack.stack.getType(), stack.stack.getData(), numTaken * recoveryFraction);
			}
		}
		
		for (CampaignFleetAPI fleet : battle.getSideFor(loser.getFleet())) {
			if (fleet.isPlayerFleet()) {
				fleet.getCargo().sort();
				break;
			}
		}
	}
	

	public CargoAPI getLoot() {
		return loot;
	}

	protected void lootWeapons(FleetMemberAPI member, float mult) {
		ShipVariantAPI variant = member.getVariant();
		List<String> remove = new ArrayList<String>();
		for (String slotId : variant.getNonBuiltInWeaponSlots()) {
			if ((float) Math.random() * mult > 0.75f) {
				String weaponId = variant.getWeaponId(slotId);
				loot.addItems(CargoAPI.CargoItemType.WEAPONS, weaponId, 1);
				remove.add(slotId);
			}
		}
		// DO NOT DO THIS - no point in removing them here since the ship is scrapped
		// and would need to clone the variant to do this right
//		for (String slotId : remove) {
//			variant.clearSlot(slotId);
//		}
		//System.out.println("Cleared variant: " + variant.getHullVariantId());
	}
	
	public void autoLoot() {
		DataForEncounterSide winner = getWinnerData();
		DataForEncounterSide loser = getLoserData();
		if (winner == null || loser == null) return;
		
		List<CampaignFleetAPI> winners = battle.getSideFor(winner.getFleet());
		WeightedRandomPicker<CampaignFleetAPI> picker = new WeightedRandomPicker<CampaignFleetAPI>();
		for (CampaignFleetAPI curr : winners) {
			picker.add(curr, curr.getFleetPoints());
		}
		for (CargoStackAPI stack : loot.getStacksCopy()) {
			if (stack.isNull() || stack.isFuelStack()) continue;
			
			CampaignFleetAPI pick = picker.pick();
			if (pick == null) break;
			
			CargoAPI winnerCargo = pick.getCargo();
			float spaceLeft = winnerCargo.getSpaceLeft();
			if (spaceLeft <= 0) {
				picker.remove(pick);
				continue;
			}
			
			float spacePerUnit = stack.getCargoSpacePerUnit();
			float maxUnits = (int) (spaceLeft / spacePerUnit);
			if (maxUnits > stack.getSize()) maxUnits = stack.getSize();
			maxUnits = Math.round(maxUnits * (Math.random() * 0.5f + 0.5f));
			winnerCargo.addItems(stack.getType(), stack.getData(), maxUnits);
		}
		
		picker.clear();
		for (CampaignFleetAPI curr : winners) {
			picker.add(curr, curr.getFleetPoints());
		}
		for (CargoStackAPI stack : loot.getStacksCopy()) {
			if (stack.isNull() || !stack.isFuelStack()) continue;
			
			CampaignFleetAPI pick = picker.pick();
			if (pick == null) break;
			
			CargoAPI winnerCargo = pick.getCargo();
			float spaceLeft = winnerCargo.getMaxCapacity() - winnerCargo.getFuel();
			if (spaceLeft <= 0) {
				picker.remove(pick);
				continue;
			}
			
			float spacePerUnit = stack.getCargoSpacePerUnit();
			float maxUnits = (int) (spaceLeft / spacePerUnit);
			if (maxUnits > stack.getSize()) maxUnits = stack.getSize();
			maxUnits = Math.round(maxUnits * (Math.random() * 0.5f + 0.5f));
			winnerCargo.addItems(stack.getType(), stack.getData(), maxUnits);
		}
	}

	public boolean hasWinnerAndLoser() {
		return getWinner() != null && getLoser() != null;
	}
	
	public CampaignFleetAPI getWinner() {
		return getWinnerData() != null ? getWinnerData().getFleet() : null;
	}
	public CampaignFleetAPI getLoser() {
		return getLoserData() != null ? getLoserData().getFleet() : null;
	}
	
	public boolean canOutrunOtherFleet(CampaignFleetAPI fleet, CampaignFleetAPI other) {
		return fleet.getFleetData().getMinBurnLevel() >= other.getFleetData().getMaxBurnLevel() + 1f;
	}
	
	public static float getWeaponSurvivialOnRepairChance() {
		return 0.1f;
	}
	
	public float getRepairChance(FleetMemberAPI member, CampaignFleetAPI fleet) {
		float result;
		PersonAPI commander = battle.getSourceFleet(member).getCommander();
		if (commander != null) {
			result = commander.getStats().getShipRepairChance().getModifiedValue() * 0.01f;
		} else {
			result = Global.getSettings().getFloat("baseShipRepairChance") * 0.01f;
		}
		
		//if (member.isFighterWing()) result += 0.5f;
		return result;
//		if (member.isFighterWing()) return 0.6f;
//		return 0.1f;
	}
	
	
	

	protected void applyResultToFleets(EngagementResultAPI result) {
//		applyCrewAndShipLosses(result);
		fixFighters(result.getWinnerResult());
		fixFighters(result.getLoserResult());
		applyShipLosses(result);	
		applyCrewLosses(result);
	}

	
	public void fixFighters(EngagementResultForFleetAPI result) {
		Set<CampaignFleetAPI> fleetsWithDecks = new HashSet<CampaignFleetAPI>();
		for (FleetMemberAPI curr : result.getReserves()) {
			if (curr.isMothballed()) continue;
			if (curr.getNumFlightDecks() > 0) {
				fleetsWithDecks.add(battle.getSourceFleet(curr));
				break;
			}
		}
		for (FleetMemberAPI curr : result.getDeployed()) {
			if (curr.isMothballed()) continue;
			if (curr.getNumFlightDecks() > 0) {
				fleetsWithDecks.add(battle.getSourceFleet(curr));
				break;
			}
		}
		for (FleetMemberAPI curr : result.getRetreated()) {
			if (curr.isMothballed()) continue;
			if (curr.getNumFlightDecks() > 0) {
				fleetsWithDecks.add(battle.getSourceFleet(curr));
				break;
			}
		}
		
		List<FleetMemberAPI> saved = new ArrayList<FleetMemberAPI>();
		for (FleetMemberAPI curr : result.getDestroyed()) {
			if (!fleetsWithDecks.contains(battle.getSourceFleet(curr))) continue;
			if (curr.isFighterWing()) {
				saved.add(curr);
			}
		}
		
		result.getDestroyed().removeAll(saved);
		result.getRetreated().addAll(saved);
		
		
		List<FleetMemberAPI> toRepair = new ArrayList<FleetMemberAPI>();
		toRepair.addAll(result.getDeployed());
		toRepair.addAll(result.getRetreated());
		for (FleetMemberAPI curr : toRepair) {
			if (curr.isFighterWing()) {
				if (fleetsWithDecks.contains(battle.getSourceFleet(curr))) {
					curr.getStatus().repairFully();
				} else {
					curr.getStatus().repairFullyNoNewFighters();
				}
			}
		}
		
	}
	
	protected void applyCrewLosses(EngagementResultAPI result) {
		EngagementResultForFleetAPI winner = result.getWinnerResult();
		EngagementResultForFleetAPI loser = result.getLoserResult();
		
		//boolean playerInvolved = winner.getFleet().isPlayerFleet() || loser.getFleet().isPlayerFleet();
		boolean playerInvolved = battle.isPlayerInvolved();
		calculateAndApplyCrewLosses(winner, playerInvolved);
		calculateAndApplyCrewLosses(loser, playerInvolved);
		
//		applyCrewLosses(winner);
//		applyCrewLosses(loser);
	}
	
	protected void applyShipLosses(EngagementResultAPI result) {
		EngagementResultForFleetAPI winner = result.getWinnerResult();
		EngagementResultForFleetAPI loser = result.getLoserResult();
		
		applyShipLosses(winner);
		applyShipLosses(loser);
		
		applyCREffect(winner);
		applyCREffect(loser);
	}
	
	protected Map<FleetMemberAPI, Float> preEngagementCRForWinner = new HashMap<FleetMemberAPI, Float>();
	protected void applyCREffect(EngagementResultForFleetAPI result) {
		boolean wonBattle = result.isWinner();
		if (wonBattle) {
			preEngagementCRForWinner.clear();
			for (FleetMemberAPI member : result.getFleet().getFleetData().getMembersListCopy()) {
				preEngagementCRForWinner.put(member, member.getRepairTracker().getCR());
			}
		}
		
		for (FleetMemberAPI member : result.getDisabled()) {
			member.getRepairTracker().applyCREvent(-1, "disabled in combat");
		}
		for (FleetMemberAPI member : result.getDestroyed()) {
			member.getRepairTracker().applyCREvent(-1, "disabled in combat");
		}
		
		for (FleetMemberAPI member : result.getDeployed()) {
			float deployCost = getDeployCost(member);
			if (member.isFighterWing()) {
				member.getRepairTracker().applyCREvent(-deployCost, "wing deployed in combat");
			} else {
				member.getRepairTracker().applyCREvent(-deployCost, "deployed in combat");
			}
			
			applyExtendedCRLossIfNeeded(result, member);
		}
		
		//float retreatLossMult = StarfarerSettings.getCRLossMultForRetreatInLoss();
		float retreatLossMult = Global.getSettings().getFloat("crLossMultForRetreatInLoss");
		for (FleetMemberAPI member : result.getRetreated()) {
			float deployCost = getDeployCost(member);
			if (member.isFighterWing()) {
				member.getRepairTracker().applyCREvent(-deployCost, "wing deployed in combat");
			} else {
				member.getRepairTracker().applyCREvent(-deployCost, "deployed in combat");
			}
			
			applyExtendedCRLossIfNeeded(result, member);
			
			if (!wonBattle && result.getGoal() != FleetGoal.ESCAPE) {
				float retreatCost = deployCost * retreatLossMult;
				if (retreatCost > 0) {
					member.getRepairTracker().applyCREvent(-retreatCost, "retreated from lost engagement");
				}
			}
		}
		
//		// important, so that in-combat Ship objects can be garbage collected.
//		// Probably some combat engine references in there, too. 
//		// NOTE: moved this elsewhere in this class
//		result.resetAllEverDeployed();
//		getDataFor(result.getFleet()).getMemberToDeployedMap().clear();
	}
	
	
	/**
	 * Only matters in non-auto-resolved battles.
	 * @param member
	 */
	protected void applyExtendedCRLossIfNeeded(EngagementResultForFleetAPI result, FleetMemberAPI member) {
		DeployedFleetMemberAPI dfm = getDataFor(result.getFleet()).getMemberToDeployedMap().get(member);
		if (dfm == null) return;
		
		if (dfm.getMember() == member && dfm.isFighterWing()) {
			float finalCR = dfm.getShip().getRemainingWingCR();
			float cr = member.getRepairTracker().getBaseCR();
			if (cr > finalCR) {
				member.getRepairTracker().applyCREvent(-(cr - finalCR), "deployed replacement chassis in combat");
			}
			return;
		}
		if (dfm.getMember() == member && !dfm.isFighterWing()) {
			float deployCost = getDeployCost(member);
			float endOfCombatCR = dfm.getShip().getCurrentCR() - deployCost;
			float cr = member.getRepairTracker().getCR();
			if (cr > endOfCombatCR) {
				member.getRepairTracker().applyCREvent(-(cr - endOfCombatCR), "extended deployment");
			}
			
			
			ShipAPI ship = dfm.getShip();
			if (dfm.getShip() != null && !dfm.isFighterWing()) {
				float wMult = Global.getSettings().getFloat("crLossMultForWeaponDisabled");
				float eMult = Global.getSettings().getFloat("crLossMultForFlameout");
				float mMult = Global.getSettings().getFloat("crLossMultForMissilesFired");
				float hMult = Global.getSettings().getFloat("crLossMultForHullDamage");
				
				float hullDamageFraction = ship.getHullLevelAtDeployment() - ship.getLowestHullLevelReached();
				float hullDamageCRLoss = hullDamageFraction * hMult;
				if (hullDamageCRLoss > 0) {
					member.getRepairTracker().applyCREvent(-hullDamageCRLoss, "hull damage sustained");
				}
				
				member.getStatus().setHullFraction(ship.getLowestHullLevelReached());
				
				float totalDisabled = 0f;
				MutableCharacterStatsAPI stats = member.getFleetCommander().getStats();
				float maxOP = ship.getVariant().getHullSpec().getOrdnancePoints(stats);
				if (maxOP <= 1) maxOP = 1;
				
				for (WeaponAPI w : ship.getDisabledWeapons()) {
					totalDisabled += w.getSpec().getOrdnancePointCost(stats) * wMult;
				}
				if (ship.getNumFlameouts() > 0) {
					totalDisabled += maxOP * eMult;
				}
				
				float damageBasedCRLoss = Math.min(1f, totalDisabled / maxOP);
				if (damageBasedCRLoss > 0) {
					member.getRepairTracker().applyCREvent(-damageBasedCRLoss, "weapon and engine damage sustained");
				}
				
				float missileReloadOP = 0f;
				for (WeaponAPI w : ship.getAllWeapons()) {
					if (w.getType() == WeaponType.MISSILE && w.usesAmmo()) {
						missileReloadOP += (1f - (float) w.getAmmo() / (float) w.getMaxAmmo()) * w.getSpec().getOrdnancePointCost(stats) * mMult;
					}
				}
				
				float missileReloadLoss = Math.min(1f, missileReloadOP / maxOP);
				if (missileReloadLoss > 0) {
					member.getRepairTracker().applyCREvent(-missileReloadLoss, "missile weapons used in combat");
				}
			}
			
			return;
		}
	}
	
	
	protected void applyShipLosses(EngagementResultForFleetAPI result) {
		for (FleetMemberAPI member : result.getDestroyed()) {
			battle.getSourceFleet(member).removeFleetMemberWithDestructionFlash(member);
			result.getFleet().getFleetData().removeFleetMember(member);
		}
		for (FleetMemberAPI member : result.getDisabled()) {
			battle.getSourceFleet(member).removeFleetMemberWithDestructionFlash(member);
			result.getFleet().getFleetData().removeFleetMember(member);
		}
	}
	
//	protected void applyCrewLosses(EngagementResultForFleetAPI result) {
//		CargoAPI cargo = result.getFleet().getCargo();
//		DataForEncounterSide data = getDataFor(result.getFleet());
//		CrewCompositionAPI crewLosses = data.getCrewLossesDuringLastEngagement();
//		
//		cargo.removeItems(CargoAPI.CargoItemType.RESOURCES, CargoAPI.CrewXPLevel.GREEN.getId(), crewLosses.getGreen());
//		cargo.removeItems(CargoAPI.CargoItemType.RESOURCES, CargoAPI.CrewXPLevel.REGULAR.getId(), crewLosses.getRegular());
//		cargo.removeItems(CargoAPI.CargoItemType.RESOURCES, CargoAPI.CrewXPLevel.VETERAN.getId(), crewLosses.getVeteran());
//		cargo.removeItems(CargoAPI.CargoItemType.RESOURCES, CargoAPI.CrewXPLevel.ELITE.getId(), crewLosses.getElite());
//		
//		cargo.removeMarines((int) crewLosses.getMarines());
//	}

	protected float computeLossFraction(FleetMemberAPI member, EngagementResultForFleetAPI result, float hullFraction, float hullDamage) {
		if (member == null && hullFraction == 0) {
			return (0.75f + (float) Math.random() * 0.25f);
		}
		
		//System.out.println("hullDamage: " + hullDamage);
		if (member.isFighterWing() && result != null) {
			//System.out.println("Fighter hullDamage: " + hullDamage);
			float extraLossMult = hullDamage;
			DeployedFleetMemberAPI dfm = getDataFor(result.getFleet()).getMemberToDeployedMap().get(member);
			if (dfm != null && dfm.getMember() == member) {
				float finalCR = dfm.getShip().getRemainingWingCR();
				float cr = member.getRepairTracker().getCR();
				if (cr > finalCR) {
					float crPer = dfm.getMember().getStats().getCRPerDeploymentPercent().computeEffective(dfm.getMember().getVariant().getHullSpec().getCRToDeploy()) / 100f;
					float extraCraftLost = (cr - finalCR) / crPer;
					float wingSize = dfm.getMember().getNumFightersInWing();
					if (extraCraftLost >= 1) {
						extraLossMult = hullDamage + extraCraftLost / wingSize;
					}
				}
			}
			return (0.25f + (float) Math.random() * 0.75f * (float) Math.random()) * member.getStats().getCrewLossMult().getModifiedValue() * extraLossMult;
		}
		
		if (hullFraction == 0) {
			return (0.75f + (float) Math.random() * 0.25f) * member.getStats().getCrewLossMult().getModifiedValue(); 
		}
		return hullDamage * hullDamage * (0.5f + (float) Math.random() * 0.5f) * member.getStats().getCrewLossMult().getModifiedValue();
	}
	
	protected float computeRecoverableFraction(FleetMemberAPI member, EngagementResultForFleetAPI result, float hullFraction, float hullDamage) {
		float f = 1f - computeLossFraction(member, result, hullFraction, hullDamage);
		if (f < 0) f = 0;
		return f;
	}
	
	public void calculateAndApplyCrewLosses(EngagementResultForFleetAPI result, boolean playerInvolved) {
		boolean wonBattle = result.isWinner(); 
		
		DataForEncounterSide data = getDataFor(result.getFleet());
		CrewCompositionAPI recoverable = data.getRecoverableCrewLosses();
		
		List<FleetMemberAPI> all = new ArrayList<FleetMemberAPI>();
		all.addAll(result.getDisabled());
		//all.addAll(captured);
		all.addAll(result.getDeployed());
		all.addAll(result.getDestroyed());
		all.addAll(result.getRetreated());
		all.addAll(result.getReserves());
		
//		float extraGreen = result.getFleet().getCargo().getCrew(CrewXPLevel.GREEN);
//		for (FleetMemberAPI member : all) {
//			extraGreen -= member.getCrewComposition().getCrew(CrewXPLevel.GREEN);
//		}

		CrewCompositionAPI playerLosses = data.getCrewLossesDuringLastEngagement();
		playerLosses.removeAllCrew();

		CrewCompositionAPI crewLosses = Global.getFactory().createCrewComposition();
		crewLosses.removeAllCrew();
		
		CampaignFleetAPI playerFleet = null;
		float maxExtraLoss = 0f;
		
		for (FleetMemberAPI member : all) {
			CrewCompositionAPI c = member.getCrewComposition();
			//float hull = member.getStatus().getHullFraction();
			float hullDamage = member.getStatus().getHullDamageTaken();
			float hullFraction = member.getStatus().getHullFraction();
			member.getStatus().resetDamageTaken();
			
			//if (hullDamage > 0 && !result.getFleet().isPlayerFleet() && playerInvolved) {
//			if (hullDamage > 0 && playerInvolved &&
//					!battle.getPlayerSide().contains(battle.getSourceFleet(member))) {
//				playerDidSeriousDamage = true;
//			}
//			if (lostBattle) {
//				System.out.println("HERE");
//			}
			
			float f1 = computeLossFraction(member, result, hullFraction, hullDamage);
			float f2 = computeLossFraction(member, result, hullFraction, hullDamage);
			float f3 = computeLossFraction(member, result, hullFraction, hullDamage);
			float f4 = computeLossFraction(member, result, hullFraction, hullDamage);
			
			// ship is disabled or destroyed, lose all crew for now, but it may be recovered later
			if (result.getDisabled().contains(member) || result.getDestroyed().contains(member)) {
				if (playerInvolved &&
						!battle.getPlayerSide().contains(battle.getSourceFleet(member))) {
					playerDidSeriousDamage = true;
				}
				if (battle.getSourceFleet(member) != null && battle.getSourceFleet(member).isPlayerFleet()) {
					if (f1 < 1) {
						recoverable.addGreen((1f - f1) * c.getGreen());
					}
					if (f2 < 1) {
						recoverable.addRegular((1f - f2) * c.getRegular());
					}
					if (f3 < 1) {
						recoverable.addVeteran((1f - f3) * c.getVeteran());
					}
					if (f4 < 1) {
						recoverable.addElite((1f - f4) * c.getElite());
					}
					playerLosses.addGreen(c.getGreen() * 1f);
					playerLosses.addRegular(c.getRegular() * 1f);
					playerLosses.addVeteran(c.getVeteran() * 1f);
					playerLosses.addElite(c.getElite() * 1f);
				}

				crewLosses.addGreen(c.getGreen() * 1f);
				crewLosses.addRegular(c.getRegular() * 1f);
				crewLosses.addVeteran(c.getVeteran() * 1f);
				crewLosses.addElite(c.getElite() * 1f);

				c.addGreen(-c.getGreen() * f1);
				c.addRegular(-c.getRegular() * f1);
				c.addVeteran(-c.getVeteran() * f1);
				c.addElite(-c.getElite() * f1);
				
				// c should now be left with the appropriate crew composition (base minus losses) to use
				// as a starting point for boarding actions
				
			} else {
				// the ship is still ok, only lose the non-recoverable casualties
				// for fighters, which can lose more than their actual max crew 
				if (f1 > 1) {
					crewLosses.addGreen((f1 - 1) * c.getGreen());
				}
				if (f2 > 1) {
					crewLosses.addGreen((f2 - 1) * c.getRegular());
				}
				if (f3 > 1) {
					crewLosses.addGreen((f3 - 1) * c.getVeteran());
				}
				if (f4 > 1) {
					crewLosses.addGreen((f4 - 1) * c.getElite());
				}
				
				// both fighters and normal ships
				c.transfer(CargoAPI.CrewXPLevel.GREEN, c.getGreen() * f1, crewLosses);
				c.transfer(CargoAPI.CrewXPLevel.REGULAR, c.getRegular() * f2, crewLosses);
				c.transfer(CargoAPI.CrewXPLevel.VETERAN, c.getVeteran() * f3, crewLosses);
				c.transfer(CargoAPI.CrewXPLevel.ELITE, c.getElite() * f4, crewLosses);
			}
			
			if (battle.getSourceFleet(member).isPlayerFleet() && 
					(crewLosses.getTotalCrew() > 0 || crewLosses.getMarines() > 0)) {
				playerFleet = battle.getSourceFleet(member);
				maxExtraLoss += member.getMaxCrew() - member.getMinCrew();
			}
			
			CargoAPI cargo = battle.getSourceFleet(member).getCargo();
			cargo.removeItems(CargoAPI.CargoItemType.RESOURCES, CargoAPI.CrewXPLevel.GREEN.getId(), crewLosses.getGreen());
			cargo.removeItems(CargoAPI.CargoItemType.RESOURCES, CargoAPI.CrewXPLevel.REGULAR.getId(), crewLosses.getRegular());
			cargo.removeItems(CargoAPI.CargoItemType.RESOURCES, CargoAPI.CrewXPLevel.VETERAN.getId(), crewLosses.getVeteran());
			cargo.removeItems(CargoAPI.CargoItemType.RESOURCES, CargoAPI.CrewXPLevel.ELITE.getId(), crewLosses.getElite());
			cargo.removeMarines((int) crewLosses.getMarines());
			crewLosses.clear();
		}
		
		if (playerFleet != null) {
			CargoAPI cargo = playerFleet.getCargo();
			float maxCrew = cargo.getMaxPersonnel();
			float crew = cargo.getTotalCrew();
			float marines = cargo.getMarines();
			float recoverableTotal = recoverable.getTotalCrew() + recoverable.getMarines();
			float total = crew + marines + recoverableTotal;
			if (total > maxCrew) {
				//float toLose = Math.min(maxExtraLoss, total - maxCrew);
				float toLose = total - maxCrew;
				if (toLose > 0) {
					recoverable.clear();
					toLose -= recoverableTotal;
					total -= recoverableTotal;
					
					if (toLose > 0 && total > 0) {
						float green = cargo.getCrew(CrewXPLevel.GREEN);
						float regular = cargo.getCrew(CrewXPLevel.REGULAR);
						float veteran = cargo.getCrew(CrewXPLevel.VETERAN);
						float elite = cargo.getCrew(CrewXPLevel.ELITE);
	
						crewLosses.clear();
						crewLosses.addGreen((int)Math.ceil(green / total * toLose));
						crewLosses.addRegular((int)Math.ceil(regular / total * toLose));
						crewLosses.addVeteran((int)Math.ceil(veteran / total * toLose));
						crewLosses.addElite((int)Math.ceil(elite / total * toLose));
						crewLosses.addMarines((int)Math.ceil(marines / total * toLose));
						
						cargo.removeItems(CargoAPI.CargoItemType.RESOURCES, CargoAPI.CrewXPLevel.GREEN.getId(), crewLosses.getGreen());
						cargo.removeItems(CargoAPI.CargoItemType.RESOURCES, CargoAPI.CrewXPLevel.REGULAR.getId(), crewLosses.getRegular());
						cargo.removeItems(CargoAPI.CargoItemType.RESOURCES, CargoAPI.CrewXPLevel.VETERAN.getId(), crewLosses.getVeteran());
						cargo.removeItems(CargoAPI.CargoItemType.RESOURCES, CargoAPI.CrewXPLevel.ELITE.getId(), crewLosses.getElite());
						cargo.removeMarines((int) crewLosses.getMarines());
						crewLosses.clear();
					}
				}
			}
		}
		
	}

	public void recoverCrew(CampaignFleetAPI fleet) {
		if (battle.isPlayerSide(battle.getSideFor(fleet))) {
			DataForEncounterSide data = getDataFor(fleet);
			CargoAPI cargo = Global.getSector().getPlayerFleet().getCargo();
			CrewCompositionAPI rec = data.getRecoverableCrewLosses();
			
			cargo.addItems(CargoAPI.CargoItemType.RESOURCES, CargoAPI.CrewXPLevel.GREEN.getId(), rec.getGreen());
			cargo.addItems(CargoAPI.CargoItemType.RESOURCES, CargoAPI.CrewXPLevel.REGULAR.getId(), rec.getRegular());
			cargo.addItems(CargoAPI.CargoItemType.RESOURCES, CargoAPI.CrewXPLevel.VETERAN.getId(), rec.getVeteran());
			cargo.addItems(CargoAPI.CargoItemType.RESOURCES, CargoAPI.CrewXPLevel.ELITE.getId(), rec.getElite());
			
			cargo.addMarines((int) rec.getMarines());
		}
	}	
}











