package com.fs.starfarer.api.impl.campaign.abilities;

import java.awt.Color;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.BattleAPI;
import com.fs.starfarer.api.campaign.CampaignFleetAPI;
import com.fs.starfarer.api.campaign.BuffManagerAPI.Buff;
import com.fs.starfarer.api.characters.AbilityPlugin;
import com.fs.starfarer.api.fleet.FleetMemberAPI;
import com.fs.starfarer.api.fleet.FleetMemberViewAPI;
import com.fs.starfarer.api.impl.campaign.ids.Abilities;
import com.fs.starfarer.api.impl.campaign.terrain.CRRecoveryBuff;
import com.fs.starfarer.api.ui.LabelAPI;
import com.fs.starfarer.api.ui.TooltipMakerAPI;
import com.fs.starfarer.api.util.Misc;

public class EmergencyBurnAbility extends BaseDurationAbility {

	public static final float SENSOR_RANGE_MULT = 0.5f;
	//public static final float DETECTABILITY_MULT = 1.25f;
	public static final float DETECTABILITY_PERCENT = 50f;
	public static final float MAX_BURN_MOD = 5f;
	//public static final float CR_COST_MULT = 1f;
	public static final float CR_COST_MULT = 0.5f;
	public static final float FUEL_USE_MULT = 1f;
	
	public static final float ACCELERATION_MULT = 4f;
	
	
	
	public String getSpriteName() {
		return Global.getSettings().getSpriteName("abilities", Abilities.EMERGENCY_BURN);
	}
	
	

	@Override
	protected String getActivationText() {
		return "Emergency burn";
	}


	@Override
	protected void activateImpl() {
//		if (entity.isInCurrentLocation()) {
//			entity.addFloatingText("Emergency burn", entity.getFaction().getBaseUIColor(), 0.5f);
//		}
		
		AbilityPlugin goDark = entity.getAbility(Abilities.GO_DARK);
		if (goDark != null && goDark.isActive()) {
			goDark.deactivate();
		}
		AbilityPlugin sb = entity.getAbility(Abilities.SUSTAINED_BURN);
		if (sb != null && sb.isActive()) {
			sb.deactivate();
		}
		
		CampaignFleetAPI fleet = getFleet();
		if (fleet == null) return;
		
		for (FleetMemberAPI member : fleet.getFleetData().getMembersListCopy()) {
			float crLoss = member.getDeployCost() * CR_COST_MULT;
			member.getRepairTracker().applyCREvent(-crLoss, "Emergency burn");
		}
		
		float cost = computeFuelCost();
		fleet.getCargo().removeFuel(cost);
	}

	@Override
	protected void applyEffect(float level) {
		if (level > 0) {
			AbilityPlugin goDark = entity.getAbility(Abilities.GO_DARK);
			if (goDark != null) goDark.forceDisable();
			AbilityPlugin sb = entity.getAbility(Abilities.SUSTAINED_BURN);
			if (sb != null) sb.forceDisable();
			
//			if (entity instanceof CampaignFleetAPI) {
//				CampaignFleetAPI fleet = (CampaignFleetAPI) entity;
//				if (fleet.getBattle() != null) {
//					deactivate();
//				}
//			}
		}
		
		CampaignFleetAPI fleet = getFleet();
		if (fleet == null) return;
		
		fleet.getStats().getSensorRangeMod().modifyMult(getModId(), 1f + (SENSOR_RANGE_MULT - 1f) * level, "Emergency burn");
		fleet.getStats().getDetectedRangeMod().modifyPercent(getModId(), DETECTABILITY_PERCENT * level, "Emergency burn");
		fleet.getStats().getFleetwideMaxBurnMod().modifyFlat(getModId(), (int)(MAX_BURN_MOD * level), "Emergency burn");
		fleet.getStats().getAccelerationMult().modifyMult(getModId(), 1f + (ACCELERATION_MULT - 1f) * level);
		
		for (FleetMemberViewAPI view : fleet.getViews()) {
			//view.getContrailColor().shift(getModId(), view.getEngineColor().getBase(), 1f, 1f, 0.25f);
			view.getContrailColor().shift(getModId(), new Color(50,50,50,155), 1f, 1f, .75f);
			//view.getContrailColor().shift(getModId(), new Color(255,100,255), 1f, 1f, 0.5f);
			//view.getContrailDurMult().shift(getModId(), 0.5f, 1f, 1f, 1f);
			//view.getContrailWidthMult().shift(getModId(), 2f, 1f, 1f, 1f);
			view.getEngineGlowSizeMult().shift(getModId(), 2f, 1f, 1f, 1f);
			view.getEngineHeightMult().shift(getModId(), 5f, 1f, 1f, 1f);
			view.getEngineWidthMult().shift(getModId(), 3f, 1f, 1f, 1f);
		}
		
		//member.getStats().getBaseCRRecoveryRatePercentPerDay()for (FleetMemberAPI member : fleet.getFleetData().getMembersListCopy()) {
		String buffId = getModId();
		float buffDur = 0.1f;
		boolean needsSync = false;
		for (FleetMemberAPI member : fleet.getFleetData().getMembersListCopy()) {
			Buff test = member.getBuffManager().getBuff(buffId);
			if (test instanceof CRRecoveryBuff) {
				CRRecoveryBuff buff = (CRRecoveryBuff) test;
				buff.setDur(buffDur);
			} else {
				member.getBuffManager().addBuff(new CRRecoveryBuff(buffId, 0f, buffDur));
				needsSync = true;
			}
		}
		
		if (needsSync || fleet.isPlayerFleet()) {
			fleet.forceSync();
		}
		
		
//		if (level > 0) {
//			SlipstreamTerrainPlugin slipstream = SlipstreamTerrainPlugin.getSlipstreamPlugin(fleet.getContainingLocation());
//			if (slipstream != null) {
//				slipstream.disrupt(fleet, 0.1f);
//			}
//		}
	}

	@Override
	protected void deactivateImpl() {
		cleanupImpl();
	}
	
	@Override
	protected void cleanupImpl() {
		CampaignFleetAPI fleet = getFleet();
		if (fleet == null) return;
		
		fleet.getStats().getSensorRangeMod().unmodify(getModId());
		fleet.getStats().getDetectedRangeMod().unmodify(getModId());
		fleet.getStats().getFleetwideMaxBurnMod().unmodify(getModId());
		fleet.getStats().getAccelerationMult().unmodify(getModId());
	}
	
	@Override
	public float getActivationDays() {
		return 0.25f;
	}

	@Override
	public float getCooldownDays() {
		return 1f;
	}

	@Override
	public float getDeactivationDays() {
		return 0.25f;
	}

	@Override
	public float getDurationDays() {
		return 1.5f;
	}

	@Override
	public boolean isUsable() {
		return super.isUsable() && 
					getFleet() != null && 
					(getFleet().isAIMode() || computeFuelCost() <= getFleet().getCargo().getFuel());
	}

	private float computeFuelCost() {
		CampaignFleetAPI fleet = getFleet();
		if (fleet == null) return 0f;
		
		float cost = fleet.getLogistics().getFuelCostPerLightYear() * FUEL_USE_MULT;
		return cost;
	}
	
	private float computeSupplyCost() {
		CampaignFleetAPI fleet = getFleet();
		if (fleet == null) return 0f;
		
		float cost = 0f;
		for (FleetMemberAPI member : fleet.getFleetData().getMembersListCopy()) {
			cost += member.getDeploymentPointsCost() * CR_COST_MULT;
		}
		return cost;
	}

	
	@Override
	public void createTooltip(TooltipMakerAPI tooltip, boolean expanded) {
		CampaignFleetAPI fleet = getFleet();
		if (fleet == null) return;
		
		Color gray = Misc.getGrayColor();
		Color highlight = Misc.getHighlightColor();
		Color fuel = Global.getSettings().getColor("progressBarFuelColor");
		Color bad = Misc.getNegativeHighlightColor();
		
		LabelAPI title = tooltip.addTitle("Emergency Burn");
//		title.highlightLast(status);
//		title.setHighlightColor(gray);

		float pad = 10f;
		
		float fuelCost = computeFuelCost();
		float supplyCost = computeSupplyCost();
		
		tooltip.addPara("Consumes %s fuel and reduces the combat readiness" +
					" of all ships, costing up to %s supplies to recover. Also prevents combat readiness recovery while active.", pad, 
					highlight,
					Misc.getRoundedValueMaxOneAfterDecimal(fuelCost),
					Misc.getRoundedValueMaxOneAfterDecimal(supplyCost));
		
		if (fuelCost > fleet.getCargo().getFuel()) {
			tooltip.addPara("Not enough fuel.", bad, pad);
		}
		
		tooltip.addPara("Increases the maximum burn level by %s." +
				" Reduces sensor range by %s and increases the range at" +
				" which the fleet can be detected by %s.", pad,
				highlight, 
				"" + (int) MAX_BURN_MOD,
				"" + (int)((1f - SENSOR_RANGE_MULT) * 100f) + "%",
				//"" + (int)((DETECTABILITY_MULT - 1f) * 100f) + "%"
				"" + (int)(DETECTABILITY_PERCENT) + "%"
		);
		tooltip.addPara("Disables \"Go Dark\" when activated.", pad);
	}

	public boolean hasTooltip() {
		return true;
	}
	
	@Override
	public boolean isTooltipExpandable() {
		return false;
	}


	@Override
	public void fleetLeftBattle(BattleAPI battle, boolean engagedInHostilities) {
		if (engagedInHostilities) {
			deactivate();
		}
	}
	
	
}





