package com.fs.starfarer.api.impl.campaign.events;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.BaseOnMessageDeliveryScript;
import com.fs.starfarer.api.campaign.BattleAPI;
import com.fs.starfarer.api.campaign.CampaignFleetAPI;
import com.fs.starfarer.api.campaign.FactionAPI;
import com.fs.starfarer.api.campaign.FleetAssignment;
import com.fs.starfarer.api.campaign.LocationAPI;
import com.fs.starfarer.api.campaign.RepLevel;
import com.fs.starfarer.api.campaign.SectorEntityToken;
import com.fs.starfarer.api.campaign.StarSystemAPI;
import com.fs.starfarer.api.campaign.comm.CommMessageAPI;
import com.fs.starfarer.api.campaign.comm.MessagePriority;
import com.fs.starfarer.api.campaign.econ.MarketAPI;
import com.fs.starfarer.api.campaign.events.CampaignEventTarget;
import com.fs.starfarer.api.characters.PersonAPI;
import com.fs.starfarer.api.impl.campaign.CoreReputationPlugin.RepActionEnvelope;
import com.fs.starfarer.api.impl.campaign.CoreReputationPlugin.RepActions;
import com.fs.starfarer.api.impl.campaign.econ.BaseMarketConditionPlugin;
import com.fs.starfarer.api.impl.campaign.fleets.FleetFactoryV2;
import com.fs.starfarer.api.impl.campaign.fleets.FleetParams;
import com.fs.starfarer.api.impl.campaign.ids.Conditions;
import com.fs.starfarer.api.impl.campaign.ids.Factions;
import com.fs.starfarer.api.impl.campaign.ids.FleetTypes;
import com.fs.starfarer.api.impl.campaign.ids.MemFlags;
import com.fs.starfarer.api.impl.campaign.ids.Strings;
import com.fs.starfarer.api.impl.campaign.ids.Tags;
import com.fs.starfarer.api.impl.campaign.shared.PersonBountyEventData;
import com.fs.starfarer.api.impl.campaign.shared.SharedData;
import com.fs.starfarer.api.util.Misc;
import com.fs.starfarer.api.util.WeightedRandomPicker;

public class PersonBountyEvent extends BaseEventPlugin {
	public static Logger log = Global.getLogger(PersonBountyEvent.class);
	
	public static enum BountyType {
		PIRATE,
		DESERTER,
	}
	
	public static enum FleetType {
		LARGE,
		SMALL,
	}
	
	private float elapsedDays = 0f;
	private float duration = 60f;
	private float bountyCredits = 0;
	private MessagePriority messagePriority;
	
	private FactionAPI faction;
	private PersonAPI person;
	private CampaignFleetAPI fleet;
	private String targetDesc;
	
	private BountyType bountyType;
	//private FleetType fleetType;
	
	private PersonBountyEventData data;
	
	private SectorEntityToken hideoutLocation = null;
	
	public void init(String type, CampaignEventTarget eventTarget) {
		super.init(type, eventTarget);
		messagePriority = MessagePriority.SECTOR;
	}
	
	@Override
	public void setParam(Object param) {
		super.setParam(param);
	}

	@Override
	public boolean allowMultipleOngoingForSameTarget() {
		return false;
	}

	public void startEvent() {
		super.startEvent();
		
		data = SharedData.getData().getPersonBountyEventData();
		data.reportStarted();
		
		pickMarketAndFaction();
		if (isDone()) return;
		
		initBountyAmount();
		
		pickHideoutLocation();
		if (isDone()) return;
		
		pickBountyType();
		initPerson();
		initTargetDesc();
		
		spawnFleet();
		if (isDone()) return;
		
		if (bountyType == BountyType.PIRATE) {
			Global.getSector().reportEventStage(this, "start_pirate", null, messagePriority, new BaseOnMessageDeliveryScript() {
				public void beforeDelivery(CommMessageAPI message) {
					message.setStarSystemId(hideoutLocation.getContainingLocation().getId());
				}
			});
		} else {
			Global.getSector().reportEventStage(this, "start_deserter", null, messagePriority, new BaseOnMessageDeliveryScript() {
				public void beforeDelivery(CommMessageAPI message) {
					message.setStarSystemId(hideoutLocation.getContainingLocation().getId());
				}
			});
		}
			
		log.info(String.format("Starting person bounty by faction [%s] for person %s", faction.getDisplayName(), person.getName().getFullName()));
	}
	
	private void pickHideoutLocation() {
		WeightedRandomPicker<SectorEntityToken> picker = new WeightedRandomPicker<SectorEntityToken>();
		Set<LocationAPI> seen = new HashSet<LocationAPI>();
		List<SectorEntityToken> planets = new ArrayList<SectorEntityToken>(Global.getSector().getEntitiesWithTag(Tags.PLANET)); 
		Collections.shuffle(planets);
		for (SectorEntityToken planet : planets) {
			if (planet.getContainingLocation().isHyperspace()) continue;
			if (planet.getMarket() != null) continue; // && !planet.getMarket().getFaction().getId().equals(Factions.PIRATES)) continue;
			
			MarketAPI nearestMarket = Misc.findNearestLocalMarket(planet, 3000, null);
			if (nearestMarket != null) {
				continue;
			}
			
			float weight = 1f;
			LocationAPI loc = planet.getContainingLocation();
			if (seen.contains(loc)) {
				weight *= 0.1f;
			}
			seen.add(loc);
			
			picker.add(planet, weight);
		}
		hideoutLocation = picker.pick();
		
		if (hideoutLocation == null) {
			endEvent();
		}
		
		//hideoutLocation = Global.getSector().getEntityById("corvus_IV");
	}
	
	

	private void pickMarketAndFaction() {
		WeightedRandomPicker<MarketAPI> picker = new WeightedRandomPicker<MarketAPI>();
		for (MarketAPI market : Global.getSector().getEconomy().getMarketsCopy()) {
			if (!data.isParticipating(market.getFactionId())) continue;
			if (market.getSize() < 3) continue;
			
			float weight = market.getSize();
			if (market.hasCondition(Conditions.MILITARY_BASE)) weight *= 2f;
			if (market.hasCondition(Conditions.HEADQUARTERS)) weight *= 2f;
			
			picker.add(market, weight);
		}
		
		if (picker.isEmpty()) {
			endEvent();
			return;
		}
		
		market = picker.pick();
		faction = market.getFaction();
		eventTarget = new CampaignEventTarget(market);
	}
	
	private void initBountyAmount() {
		int level = data.getLevel();
		float highStabilityMult = BaseMarketConditionPlugin.getHighStabilityBonusMult(market);
		
		float base = Global.getSettings().getFloat("basePersonBounty");
		float perLevel = Global.getSettings().getFloat("personBountyPerLevel");
		
		bountyCredits = (int) (base + perLevel * level * highStabilityMult);
	}
	
	private void initPerson() {
		String factionId = Factions.PIRATES;
		if (bountyType == BountyType.DESERTER) {
			factionId = market.getFactionId();
		}
		person = OfficerManagerEvent.createOfficer(Global.getSector().getFaction(factionId), 
												   (int) (5 + data.getLevel() * 1.5f));
	}
	
	private void pickBountyType() {
		WeightedRandomPicker<BountyType> picker = new WeightedRandomPicker<BountyType>();
		picker.add(BountyType.PIRATE, 10f);
		
		if (data.getLevel() >= 3) {
			picker.add(BountyType.DESERTER, 10f);
		}
		bountyType = picker.pick();
	}
	
	private void initTargetDesc() {
		targetDesc = person.getName().getFullName() + " is known to be a highly capable combat officer in command of a sizeable fleet.";
	}

	public void advance(float amount) {
		if (!isEventStarted()) return;
		if (isDone()) return;
		
		float days = Global.getSector().getClock().convertToDays(amount);
		elapsedDays += days;

		if (elapsedDays >= duration && !isDone()) {
			log.info(String.format("Ending bounty on %s by %s", person.getName().getFullName(), market.getName()));
			Global.getSector().reportEventStage(this, "expire_end", messagePriority);
			endEvent();
		}
		
		if (fleet.isInCurrentLocation() && !fleet.getFaction().getId().equals(Factions.PIRATES)) {
			fleet.setFaction(Factions.PIRATES, true);
		} else if (!fleet.isInCurrentLocation() && !fleet.getFaction().getId().equals(Factions.NEUTRAL)) {
			fleet.setFaction(Factions.NEUTRAL, true);
		}
	}
	
	private boolean ended = false;

	private void endEvent() {
		data.reportEnded();
		ended = true;
		
		if (fleet != null) {
			fleet.clearAssignments();
			if (hideoutLocation != null) {
				fleet.getAI().addAssignment(FleetAssignment.GO_TO_LOCATION_AND_DESPAWN, hideoutLocation, 1000000f, null);
			} else {
				fleet.despawn();
			}
		}
	}

	public boolean isDone() {
		return ended;
	}

	public Map<String, String> getTokenReplacements() {
		Map<String, String> map = super.getTokenReplacements();
		// this is ok because the token replacement happens right as the message is sent, not when it's received
		// so the lastBounty is the correct value for the message
		map.put("$bountyCredits", "" + (int) bountyCredits);
		map.put("$sender", faction.getDisplayName());
		
		map.put("$daysLeft", "" + (int) + Math.max(1, duration - elapsedDays));
		map.put("$targetName", person.getName().getFullName());
		if (targetDesc != null) {
			map.put("$targetDesc", targetDesc);
		}
		
		if (hideoutLocation != null) {
			if (hideoutLocation.getContainingLocation() instanceof StarSystemAPI) {
				map.put("$hideoutSystem", ((StarSystemAPI)hideoutLocation.getContainingLocation()).getBaseName());
			}
			map.put("$hideout", hideoutLocation.getName());
		}
		
		return map;
	}

	@Override
	public String[] getHighlights(String stageId) {
		
		List<String> result = new ArrayList<String>();
		addTokensToList(result, "$bountyCredits");
		
		if (stageId != null && stageId.startsWith("start_")) {
			addTokensToList(result, "$daysLeft");
		}
		
		return result.toArray(new String[0]);
	}

	@Override
	public String getEventName() {
		if (isDone()) {
			return "Wanted: " + person.getName().getFullName() + " - over";
		}
		return "Wanted: " + person.getName().getFullName() + " - " + (int) bountyCredits + Strings.C;
	}
	

	@Override
	public CampaignEventCategory getEventCategory() {
		return CampaignEventCategory.BOUNTY;
	}
	
	@Override
	public String getCurrentMessageIcon() {
		return person.getPortraitSprite();
	}
	
	@Override
	public String getCurrentImage() {
		return person.getPortraitSprite();
	}
	
	public String getEventIcon() {
		return person.getPortraitSprite();
	}

	@Override
	public void reportBattleOccurred(CampaignFleetAPI primaryWinner, BattleAPI battle) {
		if (!isEventStarted()) return;
		
		if (!battle.isPlayerInvolved()) return;
		
		CampaignFleetAPI playerFleet = Global.getSector().getPlayerFleet();
		
		if (!battle.isPlayerInvolved() || !battle.isInvolved(fleet) || battle.onPlayerSide(fleet)) {
			return;
		}

		 // didn't destroy the original flagship
		if (fleet.getFlagship() != null && fleet.getFlagship().getCaptain() == person) return;
		
		RepLevel level = playerFleet.getFaction().getRelationshipLevel(market.getFaction());
		final int payment = (int) (bountyCredits * battle.getPlayerInvolvementFraction());
		if (payment <= 0) return;
		
		if (level.isAtWorst(RepLevel.SUSPICIOUS)) {
			String reportId = "bounty_payment_end";
			if (battle.getPlayerInvolvementFraction() < 1) {
				reportId = "bounty_payment_share_end";
			}
			log.info(String.format("Paying bounty of %d from faction [%s]", (int) payment, faction.getDisplayName()));
			Global.getSector().reportEventStage(this, reportId, market.getPrimaryEntity(),
										MessagePriority.ENSURE_DELIVERY,
						new BaseOnMessageDeliveryScript() {
				public void beforeDelivery(CommMessageAPI message) {
					CampaignFleetAPI playerFleet = Global.getSector().getPlayerFleet();
					playerFleet.getCargo().getCredits().add(payment);
					Global.getSector().adjustPlayerReputation(
							new RepActionEnvelope(RepActions.PERSON_BOUNTY_REWARD, null, message, true), 
							faction.getId());
				}
			});
		} else if (level.isAtWorst(RepLevel.HOSTILE)) {
			log.info(String.format("Not paying bounty, but improving rep with faction [%s]", faction.getDisplayName()));
			Global.getSector().reportEventStage(this, "bounty_no_payment_end", market.getPrimaryEntity(),
										MessagePriority.ENSURE_DELIVERY,
						new BaseOnMessageDeliveryScript() {
				public void beforeDelivery(CommMessageAPI message) {
					Global.getSector().adjustPlayerReputation(
							new RepActionEnvelope(RepActions.PERSON_BOUNTY_REWARD, null, message, true), 
							faction.getId());
				}
			});
		} else {
			log.info(String.format("Not paying bounty or improving rep with faction [%s]", faction.getDisplayName()));
			Global.getSector().reportEventStage(this, "bounty_no_rep_end", market.getPrimaryEntity(),
										MessagePriority.ENSURE_DELIVERY);
		}
		
		data.reportSuccess();
		endEvent();
	}

	@Override
	public void reportFleetDespawned(CampaignFleetAPI fleet, FleetDespawnReason reason, Object param) {
		if (isDone()) return;
		
		if (this.fleet == fleet) {
			Global.getSector().reportEventStage(this, "other_end", market.getPrimaryEntity(), messagePriority);
			endEvent();
		}
	}
	
	
	private void spawnFleet() {
		String fleetFactionId = Factions.PIRATES;
		if (bountyType == BountyType.DESERTER) {
			fleetFactionId = market.getFactionId();
		}
		
		float level = data.getLevel();
		float qf = level / 10f;
		if (qf > 1) qf = 1;
		
		String fleetName = "";
//		if (bountyType == BountyType.PIRATE) {
//			fleetName = person.getName().getLast() + "'s" + " Armada";
//		} else {
//			fleetName = person.getName().getLast() + "'s" + " Free Company";
//		}
		fleetName = person.getName().getLast() + "'s" + " Fleet";
		
		fleet = FleetFactoryV2.createFleet(new FleetParams(
				hideoutLocation.getLocationInHyperspace(),
				null, 
				fleetFactionId,
				FleetTypes.PERSON_BOUNTY_FLEET,
				5 + level * 4, // combatPts
				0f, // freighterPts 
				0f, // tankerPts
				0f, // transportPts
				0f, // linerPts
				0f, // civilianPts 
				0f, // utilityPts
				qf, // qualityBonus
				-1f, // qualityOverride
				1f + (float) level / 10f, // officer num mult
				(int) level, // officer level bonus
				person,
				person.getStats().getLevel()
				));
		if (fleet == null) {
			endEvent();
			return;
		}

		//Misc.getSourceMarket(fleet).getStats().getDynamic().getValue(Stats.OFFICER_LEVEL_MULT);
		
		fleet.getMemoryWithoutUpdate().set(MemFlags.MEMORY_KEY_PIRATE, true);
		fleet.setNoFactionInName(true);
		fleet.setFaction(Factions.NEUTRAL, true);
		fleet.setName(fleetName);

		
		LocationAPI location = hideoutLocation.getContainingLocation();
		location.addEntity(fleet);
		fleet.setLocation(hideoutLocation.getLocation().x - 500, hideoutLocation.getLocation().y + 500);
		fleet.getAI().addAssignment(FleetAssignment.ORBIT_AGGRESSIVE, hideoutLocation, 1000000f, null);
	}
	
}






