package com.fs.starfarer.api.impl.campaign.fleets;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import org.apache.log4j.Logger;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.CampaignFleetAPI;
import com.fs.starfarer.api.campaign.FactionAPI;
import com.fs.starfarer.api.campaign.econ.MarketAPI;
import com.fs.starfarer.api.characters.PersonAPI;
import com.fs.starfarer.api.combat.ShipAPI.HullSize;
import com.fs.starfarer.api.fleet.FleetMemberAPI;
import com.fs.starfarer.api.fleet.FleetMemberType;
import com.fs.starfarer.api.fleet.ShipRolePick;
import com.fs.starfarer.api.impl.campaign.events.OfficerManagerEvent;
import com.fs.starfarer.api.impl.campaign.ids.MemFlags;
import com.fs.starfarer.api.impl.campaign.ids.Ranks;
import com.fs.starfarer.api.impl.campaign.ids.ShipRoles;
import com.fs.starfarer.api.impl.campaign.ids.Stats;
import com.fs.starfarer.api.loading.AbilitySpecAPI;
import com.fs.starfarer.api.loading.FleetCompositionDoctrineAPI;
import com.fs.starfarer.api.plugins.OfficerLevelupPlugin;
import com.fs.starfarer.api.util.Misc;
import com.fs.starfarer.api.util.WeightedRandomPicker;

public class FleetFactoryV2 {

	public static Logger log = Global.getLogger(FleetFactoryV2.class);
	
	//protected static FleetCompositionDoctrineAPI doctrine;
	
	public static CampaignFleetAPI createFleet(FleetParams params) {
		
		MarketAPI market = pickMarket(params);
		if (market == null) return null;
		
//		if (market.getName().equals("Jangala")) {
//			System.out.println("wfwdfwef");
//		}
		
		String factionId = params.factionId;
		if (params.factionIdForShipPicking != null) {
			factionId = params.factionIdForShipPicking;
		}
		CampaignFleetAPI fleet = createEmptyFleet(factionId, params.fleetType, market);
		
		FleetCompositionDoctrineAPI doctrine = fleet.getFaction().getCompositionDoctrine();
		
		float qf = market.getShipQualityFactor();
		qf += params.qualityBonus;
		if (params.qualityOverride >= 0) {
			qf = params.qualityOverride;
		}
		
		Random random = new Random();
		
		
		float combatPts = params.combatPts;
		float freighterPts = params.freighterPts;
		float tankerPts = params.tankerPts;
		float transportPts = params.transportPts;
		float linerPts = params.linerPts;
		float civilianPts = params.civilianPts;
		float utilityPts = params.utilityPts;
		
		boolean capitalOk = combatPts >= doctrine.getMinPointsForCombatCapital(); 
		boolean largeCarrierOk = combatPts >= doctrine.getMinPointsForLargeCarrier(); 

		boolean cfFail = false;
		while (combatPts > 0) {
			if (!cfFail && freighterPts > 0 && random.nextFloat() < doctrine.getCombatFreighterProbability()) {
				float pts = Math.min(freighterPts, combatPts);
				if (pts > 8) pts = 8;
				float added = addRandomCombatFreighters(pts, qf, fleet, random, market);
				combatPts -= added/2f;
				freighterPts -= added/2f;
				cfFail |= added < pts/2f;
				continue;
			}
			
			if (combatPts >= 16f) {
				add16PointGroup(qf, fleet, random, market, capitalOk, largeCarrierOk, params);
				combatPts -= 16f;
			} else if (combatPts >= 12f) {
				add12PointGroup(qf, fleet, random, market, capitalOk, params);
				combatPts -= 12f;
			} else if (combatPts >= 8f) {
				add8PointGroup(qf, fleet, random, market, capitalOk, params);
				combatPts -= 8f;
			} else {
				addRandomCombatShips(combatPts, qf, fleet, random, market, capitalOk, params);
				combatPts = 0f;
			}
		}
		
		addRandomShips(freighterPts, qf, fleet, random, market, 
				ShipRoles.FREIGHTER_SMALL,
				ShipRoles.FREIGHTER_MEDIUM,
				ShipRoles.FREIGHTER_LARGE);
		
		addRandomShips(tankerPts, qf, fleet, random, market, 
				ShipRoles.TANKER_SMALL,
				ShipRoles.TANKER_MEDIUM,
				ShipRoles.TANKER_LARGE);
		
		addRandomShips(transportPts, qf, fleet, random, market, 
				ShipRoles.PERSONNEL_SMALL,
				ShipRoles.PERSONNEL_MEDIUM,
				ShipRoles.PERSONNEL_LARGE);
		
		addRandomShips(linerPts, qf, fleet, random, market, 
				ShipRoles.LINER_SMALL,
				ShipRoles.LINER_MEDIUM,
				ShipRoles.LINER_LARGE);

		
		addRandomShips(utilityPts, qf, fleet, random, market, ShipRoles.CIV_RANDOM);
		addRandomShips(civilianPts, qf, fleet, random, market, ShipRoles.UTILITY);

		//if (market.getFaction().getCustom())
		//fleet.getMemoryWithoutUpdate().set(MemFlags.MEMORY_KEY_PIRATE, true);
		
		
		//fleet.getFleetData().setFlagship(fleet.getFleetData().getMembersListCopy().get(0));
		
		fleet.getFleetData().sort();
		
		
		float minLevel = doctrine.getOfficerLevelBase() + 
						 doctrine.getOfficerLevelPerPoint() * params.getTotalPts();
		minLevel *= market.getStats().getDynamic().getValue(Stats.OFFICER_LEVEL_MULT);
		minLevel += params.officerLevelBonus;
		minLevel -= minLevel * doctrine.getOfficerLevelVariance() * 0.5f;
		minLevel = (int) minLevel;
		if (minLevel < 1) minLevel = 1;
		
		float maxLevel = minLevel + minLevel * doctrine.getOfficerLevelVariance();
		maxLevel = (int) maxLevel;
		if (maxLevel < minLevel) maxLevel = minLevel;
		if (maxLevel > params.levelLimit) maxLevel = params.levelLimit;
		if (minLevel > params.levelLimit) minLevel = params.levelLimit;
		
		float count = (doctrine.getOfficersPerPoint() * params.getTotalPts()) * params.officerNumMult;
		count *= market.getStats().getDynamic().getValue(Stats.OFFICER_NUM_MULT);
		if (params.officerNumMult > 1 && count < 1) count = 1;
		if (count > 0 && count < 1) count = 1;
		int max = (int) Global.getSettings().getFloat("officerPlayerMax");
		if (count > max) count = max;
		
		prune(fleet, (int) (params.getTotalPts() * 6), random);
		
		List<FleetMemberAPI> members = fleet.getFleetData().getMembersListCopy();
		boolean hasShip = false;
		for (FleetMemberAPI member : members) {
			if (!member.isFighterWing()) {
				hasShip = true;
				break;
			}
		}
		if (!hasShip) {
			addRandomShips(1f, qf, fleet, random, market, ShipRoles.COMBAT_SMALL);
		}
		
		addCommanderAndOfficers((int) count, minLevel, maxLevel, fleet,  params.commander, random);
		
		
		fleet.forceSync();
		
		//FleetFactoryV2.doctrine = null;
		
		if (fleet.getFleetData().getNumMembers() <= 0 || 
				fleet.getFleetData().getNumMembers() == fleet.getNumFighters()) {
			return null;
		}
		
		for (FleetMemberAPI member : members) {
			member.getRepairTracker().setCR(member.getRepairTracker().getMaxCR());
		}
		
		return fleet;
	}
	
	public static void prune(CampaignFleetAPI fleet, int maxFP, Random random) {
		List<FleetMemberAPI> members = fleet.getFleetData().getMembersListCopy();
		WeightedRandomPicker<FleetMemberAPI> picker = new WeightedRandomPicker<FleetMemberAPI>(random);
		for (FleetMemberAPI member : members) {
			picker.add(member, 1f);
		}
		int fails = 0;
		while (fleet.getFleetPoints() > maxFP && fleet.getFleetData().getNumMembers() > 1) {
			int diff = fleet.getFleetPoints() - maxFP;
			FleetMemberAPI member = picker.pickAndRemove();
			if (member != null && member.getFleetPointCost() < diff * 2f) {
				fleet.getFleetData().removeFleetMember(member);
			} else {
				fails++;
			}
			if (fails >= 5) break;
		}
	}
	
	
	public static void addCommanderAndOfficers(int total, float minLevel, float maxLevel, CampaignFleetAPI fleet, PersonAPI commanderPerson, Random random) {
		OfficerLevelupPlugin plugin = (OfficerLevelupPlugin) Global.getSettings().getPlugin("officerLevelUp");
		int max = plugin.getMaxLevel(null);
		
		List<FleetMemberAPI> members = fleet.getFleetData().getMembersListCopy();
		
//		int level = (int) Math.min(max, Math.round(minLevel + random.nextFloat() * (maxLevel - minLevel)));
//		PersonAPI person = OfficerManagerEvent.createOfficer(fleet.getFaction(), level);
//		person.setRankId(Ranks.SPACE_COMMANDER);
//		person.setPostId(Ranks.POST_FLEET_COMMANDER);
//		fleet.setCommander(person);
		//fleet.getFleetData().setFlagship(members.get(0));
		
		
		WeightedRandomPicker<FleetMemberAPI> picker = new WeightedRandomPicker<FleetMemberAPI>(random);
		for (FleetMemberAPI member : members) {
			//if (member.isCivilian()) continue;
			if (member.isFighterWing()) continue;
			if (member.isFlagship()) continue;
			if (!member.getCaptain().isDefault()) continue;
			
			float q = member.getVariant().getQuality();
			if (member.isCivilian()) q *= 0.1f;
			float weight = (float) member.getFleetPointCost() * 0.5f + (float) member.getFleetPointCost() * q;
			picker.add(member, weight);
		}
		
		boolean commander = true;
		for (int i = 0; i < total + 1; i++) {
			FleetMemberAPI member = picker.pickAndRemove();
			if (member == null) {
				break; // out of ships that need officers
			}
			
			int level = (int) Math.min(max, Math.round(minLevel + random.nextFloat() * (maxLevel - minLevel)));
			if (Misc.isEasy()) {
				 level = (int) Math.ceil((float) level * Global.getSettings().getFloat("easyOfficerLevelMult"));
			}
			
			PersonAPI person = OfficerManagerEvent.createOfficer(fleet.getFaction(), level, true);
			
			if (commander) {
				if (commanderPerson != null) {
					person = commanderPerson;
				}
				person.setRankId(Ranks.SPACE_COMMANDER);
				person.setPostId(Ranks.POST_FLEET_COMMANDER);
				fleet.setCommander(person);
				fleet.getFleetData().setFlagship(member);
				commander = false;
			} else {
				member.setCaptain(person);
			}
		}
	}
	

	public static float add8PointGroup(float qf, 
			CampaignFleetAPI fleet, 
			Random random, 
			MarketAPI market,
			boolean capitalOk,
			FleetParams params) {
		
		FleetCompositionDoctrineAPI doctrine = fleet.getFaction().getCompositionDoctrine();
		
		float r = random.nextFloat();
		
		if (r < doctrine.getSmallCarrierProbability()) {
			float cost = addToFleet(ShipRoles.CARRIER_SMALL, market, qf, random, fleet);
			return cost + addRandomFighters(8f - cost, qf, fleet, random, market, params);
		} else {
			return addRandomCombatShips(8f, qf, fleet, random, market, capitalOk, params);
		}
	}
	
	public static float add12PointGroup(float qf, 
			CampaignFleetAPI fleet, 
			Random random, 
			MarketAPI market,
			boolean capitalOk,
			FleetParams params) {
		
		FleetCompositionDoctrineAPI doctrine = fleet.getFaction().getCompositionDoctrine();
		
		float r = random.nextFloat();
		
		if (r < doctrine.getMediumCarrierProbability()) {
			float cost = addToFleet(ShipRoles.CARRIER_MEDIUM, market, qf, random, fleet);
			return cost + addRandomFighters(12f - cost, qf, fleet, random, market, params);
		} else {
			return addRandomCombatShips(12f, qf, fleet, random, market, capitalOk, params);
		}
	}
	
	public static float add16PointGroup(float qf, 
			CampaignFleetAPI fleet, 
			Random random, 
			MarketAPI market,
			boolean capitalOk,
			boolean largeCarrierOk,
			FleetParams params) {
		
		FleetCompositionDoctrineAPI doctrine = fleet.getFaction().getCompositionDoctrine();
		
		float r = random.nextFloat();
		
		if (r < doctrine.getLargeCarrierProbability() && largeCarrierOk) {
			float cost = addToFleet(ShipRoles.CARRIER_LARGE, market, qf, random, fleet);
			return cost + addRandomFighters(16f - cost, qf, fleet, random, market, params);
		} else {
			return addRandomCombatShips(16f, qf, fleet, random, market, capitalOk, params);
		}
	}
	
	
	public static float addToFleet(String role, MarketAPI market, float qf, Random random, CampaignFleetAPI fleet) {
		float total = 0f;
		List<ShipRolePick> picks = market.pickShipsForRole(role, fleet.getFaction().getId(), qf, random, null);
		for (ShipRolePick pick : picks) {
			total += addToFleet(pick, fleet);
		}
		return total;
	}
	
	public static float addRandomCombatShips(float pts, float qf, 
											CampaignFleetAPI fleet, 
											Random random, 
											MarketAPI market,
											boolean capitalOk,
											FleetParams params) {
		
		FleetCompositionDoctrineAPI doctrine = fleet.getFaction().getCompositionDoctrine();
		
		float picked = 0;
		int numEmpty = 0;
		while (picked < pts) {
			WeightedRandomPicker<String> roles = new WeightedRandomPicker<String>(random);
			roles.add(ShipRoles.COMBAT_SMALL, doctrine.getSmall());
			roles.add(ShipRoles.FAST_ATTACK, doctrine.getFast());
			
			if (pts - picked >= 3) {
				roles.add(ShipRoles.INTERCEPTOR, doctrine.getInterceptors());
				roles.add(ShipRoles.FIGHTER, doctrine.getFighters());
				float bomberMult = 1f;
				if (pts - picked < 4) {
					bomberMult = 0.25f;
				}
				roles.add(ShipRoles.BOMBER, doctrine.getBombers() * bomberMult);
			}
			
			if (pts - picked >= 4) {
				roles.add(ShipRoles.COMBAT_MEDIUM, doctrine.getMedium());
				roles.add(ShipRoles.ESCORT_SMALL, doctrine.getSmall() * doctrine.getEscortSmallFraction());
			}
			
			if (pts - picked >= 7) {
				roles.add(ShipRoles.COMBAT_LARGE, doctrine.getLarge());
				roles.add(ShipRoles.ESCORT_MEDIUM, doctrine.getMedium() * doctrine.getEscortMediumFraction());
			}
			
			if (pts - picked >= 10 && capitalOk) {
				roles.add(ShipRoles.COMBAT_CAPITAL, doctrine.getCapital());
			}
		
			List<ShipRolePick> picks = market.pickShipsForRole(roles.pick(), fleet.getFaction().getId(), qf, random, null);
			if (picks.isEmpty()) {
				numEmpty++;
				if (numEmpty > 5) break;
				continue;
			}
			numEmpty = 0;
			
			for (ShipRolePick pick : picks) {
				picked += addToFleet(pick, fleet);
			}
		}
		return picked;
	}
	
	
	public static float addRandomFighters(float pts, float qf, 
							CampaignFleetAPI fleet, 
							Random random, 
							MarketAPI market,
							FleetParams params) {

		FleetCompositionDoctrineAPI doctrine = fleet.getFaction().getCompositionDoctrine();
		
		float picked = 0;
		int numEmpty = 0;
		while (picked < pts) {
			WeightedRandomPicker<String> roles = new WeightedRandomPicker<String>(random);
			roles.add(ShipRoles.INTERCEPTOR, doctrine.getInterceptors());
			roles.add(ShipRoles.FIGHTER, doctrine.getFighters());
			roles.add(ShipRoles.BOMBER, doctrine.getBombers());
			
			List<ShipRolePick> picks = market.pickShipsForRole(roles.pick(), fleet.getFaction().getId(), qf, random, null);
			if (picks.isEmpty()) {
				numEmpty++;
				if (numEmpty > 5) break;
				continue;
			}
			numEmpty = 0;

			for (ShipRolePick pick : picks) {
				picked += addToFleet(pick, fleet);
			}
		}
		return picked;
	}
	
	public static float addRandomCombatFreighters(float pts, float qf, 
			CampaignFleetAPI fleet, 
			Random random, 
			MarketAPI market) {

		FleetCompositionDoctrineAPI doctrine = fleet.getFaction().getCompositionDoctrine();
		
		float picked = 0;
		int numEmpty = 0;
		while (picked < pts) {
			WeightedRandomPicker<String> roles = new WeightedRandomPicker<String>(random);
			roles.add(ShipRoles.COMBAT_FREIGHTER_SMALL, doctrine.getSmall());
			if (pts - picked >= 2) {
				roles.add(ShipRoles.COMBAT_FREIGHTER_MEDIUM, doctrine.getMedium());
			}
			if (pts - picked >= 4) {
				roles.add(ShipRoles.COMBAT_FREIGHTER_LARGE, doctrine.getLarge());
			}

			List<ShipRolePick> picks = market.pickShipsForRole(roles.pick(), fleet.getFaction().getId(), qf, random, null);
			if (picks.isEmpty()) {
				numEmpty++;
				if (numEmpty > 5) break;
				continue;
			}
			numEmpty = 0;

			for (ShipRolePick pick : picks) {
				picked += addToFleet(pick, fleet);
			}
		}
		return picked;
	}
	
	public static float addRandomShips(float pts, float qf, 
			CampaignFleetAPI fleet, 
			Random random, 
			MarketAPI market,
			String smallRole, String mediumRole, String largeRole) {
		if (random == null) random = new Random();
		FleetCompositionDoctrineAPI doctrine = fleet.getFaction().getCompositionDoctrine();
		
		if (pts <= 0) return 0f;
		
		float picked = 0;
		int numEmpty = 0;
		while (picked < pts) {
			WeightedRandomPicker<String> roles = new WeightedRandomPicker<String>(random);
			if (smallRole != null) {
				roles.add(smallRole, doctrine.getSmall());
			}

			if (pts - picked >= 2 && mediumRole != null) {
				roles.add(mediumRole, doctrine.getMedium());
			}

			if (pts - picked >= 6 && largeRole != null) {
				roles.add(largeRole, doctrine.getLarge());
			}

			List<ShipRolePick> picks = market.pickShipsForRole(roles.pick(), fleet.getFaction().getId(), qf, random, null);
			if (picks.isEmpty()) {
				numEmpty++;
				if (numEmpty > 5) break;
				continue;
			}
			numEmpty = 0;

			for (ShipRolePick pick : picks) {
				picked += addToFleet(pick, fleet);
			}
		}
		return picked;
	}
	
	public static float addRandomShips(float pts, float qf, 
			CampaignFleetAPI fleet, 
			Random random, 
			MarketAPI market,
			String role) {

		if (pts <= 0) return 0f;
		
		float picked = 0;
		int numEmpty = 0;
		while (picked < pts) {

			List<ShipRolePick> picks = market.pickShipsForRole(role, fleet.getFaction().getId(), qf, random, null);
			if (picks.isEmpty()) {
				numEmpty++;
				if (numEmpty > 5) break;
				continue;
			}
			numEmpty = 0;

			for (ShipRolePick pick : picks) {
				picked += addToFleet(pick, fleet);
			}
		}
		return picked;
	}
	
	
	protected static float addToFleet(ShipRolePick pick, CampaignFleetAPI fleet) {
		FleetMemberType type = FleetMemberType.SHIP;
		if (pick.isFighterWing()) {
			type = FleetMemberType.FIGHTER_WING;
		}
		fleet.getFleetData().addFleetMember(Global.getFactory().createFleetMember(type, pick.variantId));
		return getPointsForVariant(pick.variantId);
	}
	
	public static float getPointsForVariant(String variantId) {
		if (variantId.endsWith("_wing")) return 1;
		
		HullSize size = Global.getSettings().getVariant(variantId).getHullSize();
		switch (size) {
		case CAPITAL_SHIP: return 8;
		case CRUISER: return 4;
		case DESTROYER: return 2;
		case FIGHTER:
		case FRIGATE:
			return 1;
		
		}
		return 1;
	}
	
	
	
	
	
	public static MarketAPI pickMarket(FleetParams params) {
		if (params.market != null) return params.market;
		
		if (params.hyperspaceLocation == null) return null;
		
		List<MarketAPI> allMarkets = Global.getSector().getEconomy().getMarketsCopy();
		
		int size = getMinPreferredMarketSize(params);
		float distToClosest = Float.MAX_VALUE;
		MarketAPI closest = null;
		float distToClosestMatchingSize = Float.MAX_VALUE;
		MarketAPI closestMatchingSize = null;
		
		
		for (MarketAPI market : allMarkets) {
			if (market.getPrimaryEntity() == null) continue;
			if (!market.getFactionId().equals(params.factionId)) continue;
			
			float currDist = Misc.getDistance(market.getPrimaryEntity().getLocationInHyperspace(),
											  params.hyperspaceLocation);
			if (currDist < distToClosest) {
				distToClosest = currDist;
				closest = market;
			}
			
			if (market.getSize() >= size && currDist < distToClosestMatchingSize) {
				distToClosestMatchingSize = currDist;
				closestMatchingSize = market;
			}
		}
		
		if (closestMatchingSize != null) {
			return closestMatchingSize;
		}
		
		if (closest != null) {
			return closest;
		}
		
//		MarketAPI temp = Global.getFactory().createMarket("temp", "Temp", size);
//		temp.setFactionId(params.factionId);
//		return temp;
		return null;
	}
	
	public static int getMinPreferredMarketSize(FleetParams params) {
		float fp = params.getTotalPts();
		
		if (fp <= 20) return 1;
		if (fp <= 50) return 3;
		if (fp <= 100) return 5;
		if (fp <= 150) return 7;
		
		return 8;
	}
	
	
	
	
	private static List<String> startingAbilities = null;
	public static CampaignFleetAPI createEmptyFleet(String factionId, String fleetType, MarketAPI market) {
		FactionAPI faction = Global.getSector().getFaction(factionId);
		String fleetName = faction.getFleetTypeName(fleetType); 
		CampaignFleetAPI fleet = Global.getFactory().createEmptyFleet(factionId, fleetName, true);
		fleet.getMemoryWithoutUpdate().set(MemFlags.MEMORY_KEY_FLEET_TYPE, fleetType);
		
		if (market != null) {
			fleet.getMemoryWithoutUpdate().set(MemFlags.MEMORY_KEY_SOURCE_MARKET, market.getId());
		}
		
		if (startingAbilities == null) {
			startingAbilities = new ArrayList<String>();
			for (String id : Global.getSettings().getSortedAbilityIds()) {
				AbilitySpecAPI spec = Global.getSettings().getAbilitySpec(id);
				if (spec.isAIDefault()) {
					startingAbilities.add(id);
				}
			}
		}
		
		for (String id : startingAbilities) {
			fleet.addAbility(id);
		}
		
		return fleet;
	}
	
}
