package com.fs.starfarer.api.impl.campaign.submarkets;

import org.apache.log4j.Logger;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.CargoAPI;
import com.fs.starfarer.api.campaign.CargoStackAPI;
import com.fs.starfarer.api.campaign.CoreUIAPI;
import com.fs.starfarer.api.campaign.FactionAPI;
import com.fs.starfarer.api.campaign.CargoAPI.CargoItemType;
import com.fs.starfarer.api.campaign.econ.CommodityOnMarketAPI;
import com.fs.starfarer.api.campaign.econ.SubmarketAPI;
import com.fs.starfarer.api.campaign.events.CampaignEventManagerAPI;
import com.fs.starfarer.api.campaign.events.EventProbabilityAPI;
import com.fs.starfarer.api.impl.campaign.ids.Commodities;
import com.fs.starfarer.api.impl.campaign.ids.Events;
import com.fs.starfarer.api.impl.campaign.ids.ShipRoles;
import com.fs.starfarer.api.util.Highlights;
import com.fs.starfarer.api.util.Misc;
import com.fs.starfarer.api.util.WeightedRandomPicker;

public class BlackMarketPlugin extends BaseSubmarketPlugin {
	
	public static Logger log = Global.getLogger(BlackMarketPlugin.class);
	
	public void init(SubmarketAPI submarket) {
		super.init(submarket);
	}


	public void updateCargoPrePlayerInteraction() {
		
		if (!okToUpdateCargo()) return;
		sinceLastCargoUpdate = 0f;
		
		CargoAPI cargo = getCargo();
		float stability = market.getStabilityValue();
		float blackMarketLegalFraction = 0.11f - 0.01f * stability;
		float blackMarketIllegalFraction = 0.2f - 0.01f * stability;
		
		for (CommodityOnMarketAPI com : market.getAllCommodities()) {
			String id = com.getId();
			boolean illegal = market.isIllegal(id);
			
			float desired;
			float basedOnSupply = 0f;
			if (illegal) {
				//desired = computeStockpileAvailableForSale(com) * blackMarketIllegalFraction;
				desired = com.getAverageStockpileAfterDemand() * blackMarketIllegalFraction;
				basedOnSupply = com.getSupplyValue() * blackMarketIllegalFraction;
			} else {
				desired = com.getAverageStockpileAfterDemand() * blackMarketLegalFraction;
				basedOnSupply = com.getSupplyValue() * blackMarketLegalFraction;
			}
			
			desired = (int) Math.max(desired, basedOnSupply);

			if (id.equals(Commodities.FUEL)) {
				desired = Math.max(desired, Global.getSettings().getFloat("blackMarketMinFuel"));
			} else if (id.equals(Commodities.SUPPLIES)) {
				desired = Math.max(desired, Global.getSettings().getFloat("blackMarketMinSupplies"));
			} else if (id.equals(Commodities.MARINES)) {
				desired = Math.max(desired, Global.getSettings().getFloat("blackMarketMinMarines"));
			}
			
			
			float current = cargo.getQuantity(CargoItemType.RESOURCES, id);
			if (desired > current) {
				cargo.addItems(CargoItemType.RESOURCES, id, (int)(desired - current));
			} else if (current > desired) {
				cargo.removeItems(CargoItemType.RESOURCES, id, (int)(current - desired));
			}
		}
		
		pruneWeapons(0.5f);
		
		WeightedRandomPicker<FactionAPI> factionPicker = new WeightedRandomPicker<FactionAPI>();
		factionPicker.add(market.getFaction(), 10f - stability);
		factionPicker.add(submarket.getFaction(), 10f);
		addWeaponsBasedOnMarketSize(3, 3, factionPicker);
		
		addRandomWeapons(Math.max(1, market.getSize() - 3), 3);
	
		addShips();
		cargo.sort();
		
	}
	
	private void addShips() {
		int marketSize = market.getSize();
		
		pruneShips(0.75f);
		
		WeightedRandomPicker<String> rolePicker = new WeightedRandomPicker<String>();
		rolePicker.add(ShipRoles.CIV_RANDOM, 5f);
		rolePicker.add(ShipRoles.FREIGHTER_SMALL, 3f);
		rolePicker.add(ShipRoles.TANKER_SMALL, 1f);
		rolePicker.add(ShipRoles.COMBAT_FREIGHTER_SMALL, 15f);
		rolePicker.add(ShipRoles.COMBAT_SMALL, 15f);
		rolePicker.add(ShipRoles.COMBAT_MEDIUM, 15f);
		rolePicker.add(ShipRoles.INTERCEPTOR, 5f);
		rolePicker.add(ShipRoles.FIGHTER, 5f);
		rolePicker.add(ShipRoles.BOMBER, 5f);
		rolePicker.add(ShipRoles.LINER_SMALL, 1f);
		
		if (marketSize >= 4) {
			rolePicker.add(ShipRoles.FREIGHTER_MEDIUM, 5f);
			rolePicker.add(ShipRoles.TANKER_MEDIUM, 1f);
			rolePicker.add(ShipRoles.COMBAT_FREIGHTER_MEDIUM, 10f);
			rolePicker.add(ShipRoles.COMBAT_MEDIUM, 10f);
			rolePicker.add(ShipRoles.CARRIER_SMALL, 10f);
			rolePicker.add(ShipRoles.LINER_MEDIUM, 1f);
		}
		
		if (marketSize >= 6) {
			rolePicker.add(ShipRoles.FREIGHTER_LARGE, 3f);
			rolePicker.add(ShipRoles.TANKER_LARGE, 1f);
			rolePicker.add(ShipRoles.COMBAT_FREIGHTER_LARGE, 5f);
			rolePicker.add(ShipRoles.COMBAT_LARGE, 5f);
			rolePicker.add(ShipRoles.CARRIER_MEDIUM, 10f);
			rolePicker.add(ShipRoles.LINER_LARGE, 1f);
		}
		
		WeightedRandomPicker<FactionAPI> factionPicker = new WeightedRandomPicker<FactionAPI>();
		float stability = market.getStabilityValue();
		factionPicker.add(market.getFaction(), 10f - stability);
		factionPicker.add(submarket.getFaction(), 10f);
		
		addShipsForRoles(marketSize * 2, rolePicker, factionPicker);
	}
	

	@Override
	public boolean isIllegalOnSubmarket(CargoStackAPI stack, TransferAction action) {
		return false;
	}


	@Override
	public boolean isIllegalOnSubmarket(String commodityId, TransferAction action) {
		return false;
	}
	
	public float getTariff() {
		return 0f;
	}


	@Override
	public boolean isBlackMarket() {
		return true;
	}
	
	
	public String getTooltipAppendix(CoreUIAPI ui) {
		if (isEnabled(ui)) {
			CampaignEventManagerAPI manager = Global.getSector().getEventManager();
			EventProbabilityAPI ep = manager.getProbability(Events.INVESTIGATION_SMUGGLING, market);
			float p = ep.getProbability();
			if (p < 0.01f) return "Suspicion level: none";
			
			if (p < 0.05f) {
				return "Suspicion level: minimal";
			}
			if (p < 0.1f) {
				return "Suspicion level: medium";
			}
			if (p < 0.25f) {
				return "Suspicion level: high";
			}
			if (p < 0.5f) {
				return "Suspicion level: very high";
			}
			return "Suspicion level: extreme";
		}

		return null;
	}
	
	public Highlights getTooltipAppendixHighlights(CoreUIAPI ui) {
		String appendix = getTooltipAppendix(ui);
		if (appendix == null) return null;
		
		Highlights h = new Highlights();
		h.setText(appendix);
		h.setColors(Misc.getNegativeHighlightColor());
		return h;
	}
}



