package com.fs.starfarer.api.impl.campaign.submarkets;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.Script;
import com.fs.starfarer.api.campaign.CampaignFleetAPI;
import com.fs.starfarer.api.campaign.CargoStackAPI;
import com.fs.starfarer.api.campaign.CoreUIAPI;
import com.fs.starfarer.api.campaign.econ.SubmarketAPI;
import com.fs.starfarer.api.util.Highlights;
import com.fs.starfarer.api.util.Misc;

public class StoragePlugin extends BaseSubmarketPlugin {
	
	private boolean playerPaidToUnlock = false;
	
	public void init(SubmarketAPI submarket) {
		super.init(submarket);
	}

	public void updateCargoPrePlayerInteraction() {

	}

	@Override
	public boolean isIllegalOnSubmarket(CargoStackAPI stack, TransferAction action) {
		return super.isIllegalOnSubmarket(stack, action);
	}

	@Override
	public boolean isIllegalOnSubmarket(String commodityId, TransferAction action) {
		return super.isIllegalOnSubmarket(commodityId, action);
	}
	
	public boolean isParticipatesInEconomy() {
		return false;
	}
	
	public float getTariff() {
		return 0f;
	}

	@Override
	public boolean isFreeTransfer() {
		return true;
	}

	@Override
	public String getBuyVerb() {
		return "Take";
	}

	@Override
	public String getSellVerb() {
		return "Leave";
	}

	public String getIllegalTransferText(CargoStackAPI stack, TransferAction action) {
		return "Illegal to put into storage here";
	}

//	@Override
//	public void reportPlayerMarketTransaction(PlayerMarketTransaction transaction) {
//		// do nothing; don't want to adjust market stockpiles since player storage
//		// doesn't affect those
//		// don't need to do this since isParticipatesInEconomy() returns false
//	}
	
	
	
	public boolean isEnabled(CoreUIAPI ui) {
		//if (mode == CoreUITradeMode.SNEAK) return false;
		//return playerPaidToUnlock;
		return true;
	}
	
	public void setPlayerPaidToUnlock(boolean playerPaidToUnlock) {
		this.playerPaidToUnlock = playerPaidToUnlock;
	}

	public OnClickAction getOnClickAction(CoreUIAPI ui) {
		if (playerPaidToUnlock) return OnClickAction.OPEN_SUBMARKET;
		return OnClickAction.SHOW_TEXT_DIALOG;
	}
	
	private int getUnlockCost() {
		return 5000;
	}
	
	private boolean canPlayerAffordUnlock() {
		CampaignFleetAPI playerFleet = Global.getSector().getPlayerFleet();
		int credits = (int) playerFleet.getCargo().getCredits().get();
		return credits >= getUnlockCost();
	}
	
	public String getDialogText(CoreUIAPI ui) {
		if (canPlayerAffordUnlock()) {
			return "Gaining access to storage on this market requires a one-time fee of " + getUnlockCost() + " credits.";
		} else {
			return "Gaining access to storage on this market requires a one-time fee of " + getUnlockCost() + " credits, which you can't afford.";
		}
	}
	public Highlights getDialogTextHighlights(CoreUIAPI ui) {
		Highlights h = new Highlights();
		h.setText("" + getUnlockCost());
		if (canPlayerAffordUnlock()) {
			h.setColors(Misc.getHighlightColor());
		} else {
			h.setColors(Misc.getNegativeHighlightColor());
		}
		return h;
	}
	public DialogOption [] getDialogOptions(CoreUIAPI ui) {
		if (canPlayerAffordUnlock()) {
			return new DialogOption [] {
				new DialogOption("Pay", new Script() {
					public void run() {
						CampaignFleetAPI playerFleet = Global.getSector().getPlayerFleet();
						playerFleet.getCargo().getCredits().subtract(getUnlockCost());
						playerPaidToUnlock = true;
					}
				}),
				new DialogOption("Never mind", null)
			};
		} else {
			return new DialogOption [] {
					new DialogOption("Never mind", null)
			};
		}
	}
	public String getTooltipAppendix(CoreUIAPI ui) {
		if (!playerPaidToUnlock) {
			return "Requires a one-time access fee of " + getUnlockCost() + " credits.";
		}
		return null;
	}
	public Highlights getTooltipAppendixHighlights(CoreUIAPI ui) {
		String appendix = getTooltipAppendix(ui);
		if (appendix == null) return null;
		
		Highlights h = new Highlights();
		h.setText("" + getUnlockCost());
		if (canPlayerAffordUnlock()) {
			h.setColors(Misc.getHighlightColor());
		} else {
			h.setColors(Misc.getNegativeHighlightColor());
		}
		return h;
	}
	
	
	
}





